#include "Tissue.hh"

#include "BCell.hh"

//#define CORRECTEDDIVSIZEGRADIENT
//#define OLDDIVSIZEGRADIENT


//to reduce sliding:
//#define GROWTHDIVSIMANDSIZECOMP//if cellheight>Hdiv, you can divide but in same step also grow
                               //also for division time use t=1/(R*(H+1)) this way if in one file
                               //cells divide and in another not you do not have this issue of
                               //divided cells being (H-1)/2 instead of H/2
                               //together this limits sliding in the division zone
//#define COMPENSATESIZEDIFFERENCESEZ//for elongation rates we compute time for next elongation as
                                   //t=1/(R*(H-((H/HMZ)-1)*1.5), giving a penalty for how often
                                   //a meristematic cell size (HMZ) fits in the cell size and hence 
                                   //compensating for the number of walls small HMZ sized cells would
                                   //have that do not contribute to growth rate, the -1 is such that
                                   //a HMZ sized cell has no penalty, only bigger cells, the *1.5 is
                                   //to compensate for the fact that while n small cells all grow once
                                   //the big cell they (together with their walls) fit into grows n
                                   //times, each time increasing its size by 1
                                   //not sure if this 1.5 is perfect, still trying that out
                
//#define SYNCEDEZENTRY//cells are only allowed to enter the EZ if all their neighbors are ready to do the
                     //the same (cell are ready if their CellMinJ>yLRC1)
                     //so if e.g. a large cell is neighbored by 3 small cells, you prevent that the top 2 
                     //small cells already start expanding rapidly while the bottom one and the large 
                     //one next to them are still growing cytoplasmically
                     //again not sure if this will help
                     //note that cells need an extra property "goodtogotoEZ" for this routine
                     

//also important: in route Growth make sure that both when you compute the rates and when you do the actual
//growth, the elongation can proceed as long as Difff<85, that is you do not stop elongation if H>=Hmax
//otherwise the primed cells that arrive largest reach this size the earliest and then stop elongating
//causing other cells to slide past

void Tissue::GrowCell(BCell *c,int time)
{
  int oldh;
  int growthtype;
  int istart,istop;
  int jstart,jstop;
  int i,j;
  BCell *cc;

  oldh=c->CellHeight;

  //shift all that lies above, and upper membrane of cell itself upwards
  istart=c->CellMinI-1;
  istop=c->CellMaxI+1;
  jstart=c->CellMaxJ;//top membrane
  //find uppermost j position within this cell file
  i=c->CellMinI+1;
  j=m-1;
  while(Tc[i][j]==3)
    j--;
  jstop=j+1;//j;//j+1;//ERROR perhaps should be j?

  cc=NULL;
  for(j=jstop;j>=jstart;j--)//j>=jstart so including topmembrane cell q
    for(i=istart;i<=istop;i++)
      {
	//if it is left cell, if the point is on the right wall
	//and there is a cell right of that right wall: do not
	//do anything and leave it to the right cell
	//similarly if it is a right cell, if the point is on the left
	//wall and there is a cell left of that left wall: do not
   	//do anything and leave it to the left cell
	if((c->CellMaxI<(n/2-1) && i==istop) && Tc[istop][j+1]==0)
	  ;
	else if((c->CellMaxI>(n/2-1) && i==istart) && Tc[istart][j+1]==0)
	  ;
	else
	  {
	    //Tc: shift cell, membrane, wall locations;
	    //Tct, Tmt: shift celltypes, membrane types;
	    //CellGrid: shift were cells are at grid
	    Tc[i][j+1]=Tc[i][j];
	    Tct[i][j+1]=Tct[i][j];
	    Tmt[i][j+1]=Tmt[i][j];
	    CellGrid[i][j+1]=CellGrid[i][j];
	    //shift Pin/Lax3/Lax2/Aux1
	    PinPrePat[i][j+1]=PinPrePat[i][j];
	    Aux1PrePat[i][j+1]=Aux1PrePat[i][j];
	    Lax3PrePat[i][j+1]=Lax3PrePat[i][j];
	    //shift auxin levels
	    Caux1[i][j+1]=Caux1[i][j];


	    //not in old program, to be more efficient
	    //instead of finding cellcorners and cellsizes for all cells again
	    if(CellGrid[i][j+1]!=NULL)//cell there
	      {
		if(CellGrid[i][j+1]!=cc)//new cell there
		  {
		    cc=CellGrid[i][j+1];//assign as current cell
		    if(cc!=c)//GAAT DIT GOED???
		      {
			cc->CellMinJ++;//cell is being shifted so j
			cc->CellMaxJ++;//positions should be done+1
			//all else stays same: i positions, width
			//height, sizes
		      }
		  }
	      }
	  }
      }

  //add extra row to cell, mind membranes, PINs etc
  j=c->CellMaxJ-1;
  for(i=istart;i<=istop;i++)
    {
      //newly aroson gap is now filled with extra row by copying below row
      Tc[i][j+1]=Tc[i][j];
      Tct[i][j+1]=Tct[i][j];
      Tmt[i][j+1]=Tmt[i][j];
      CellGrid[i][j+1]=CellGrid[i][j];
      //correct Pin/Lax3/Lax2/Aux1 in extra lateral membrane?! (dilute??)
      PinPrePat[i][j+1]=PinPrePat[i][j];
      Aux1PrePat[i][j+1]=Aux1PrePat[i][j];
      Lax3PrePat[i][j+1]=Lax3PrePat[i][j];
    }


  //growing cell is only one that changes in height and hence size
  //not in old program, to be more efficient
  //instead of finding cellcorners and cellsizes for all cells again
  c->CellMaxJ++;
  c->CellHeight++;
  c->CellSize+=c->CellWidth;
  c->InvCellSize=1./c->CellSize;

  //DILUTE AUXIN, otherwise auxin created through growth
  for(i=istart;i<=istop;i++)
    for(j=c->CellMinJ;j<=c->CellMaxJ;j++)
      {
	Caux1[i][j]*=oldh/(oldh+1.);
      }
  //dilute concentrations of proteins if necessary
  if(c->CellZonationState<60)
    growthtype=0;//cytoplasmic
  else
    growthtype=1;//vacuolar
  c->DiluteCell(c,growthtype,oldh);

  //bookkeeping of growth times
  c->TimePrevGrowth=time;


  //restrict to relevant cell file
  //so also different and more efficient then in old program
  int meani=(int)(0.5*(c->CellMinI+c->CellMaxI));
  int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
  CullOverflowCells(meani);
#ifndef DEBUGMODE
#ifndef SMBMUTANT
  //we only need to check for the need for lateral root
  //cap culling if growth occurs in this file!!!!
  if(Tct[meani][meanj]==1)//lrc1 so root cap culling relevant
    {
      if(meani<n/2)
	CullLateralRootcap(0);
      else
	CullLateralRootcap(1);
    }
#endif
#endif
}



void Tissue::DivideCell(BCell *c, int time)
{
  BCell *cnew;
  int i,j;
  double divtime1,divtime2;
  double sumdiv,sumelong;
  double divrate,elongrate;
  double elongtime1,elongtime2;
  int istart,istop,jwall;


  //first make new cell (need id and pointer)
  cnew=new BCell();
  maxidnr++;
  actcells++;
  cnew->CreateCell(maxidnr);

  //printf("cell %i will divide, created new cell with id %i, nr cells now %i\n",c->cellid,cnew->cellid,actcells);


  //#ifdef TESTING
  //insert wall, membranes and PINs on grid and
  //divide grid points over two cells
  //cell c is up for a division
  istart=c->CellMinI;
  istop=c->CellMaxI;
  jwall=c->CellHeight/2;

  for(i=istart;i<=istop;i++)
    {
      //make wall:
      j=c->CellMinJ+jwall;//wall
      Tc[i][j]=0;//wall
      Tct[i][j]=20;//wall
      Tmt[i][j]=0;//wall
      CellGrid[i][j]=NULL;//wall

      //make membrane above wall: bottom membrane top daughter cell
      j=c->CellMinJ+jwall+1;//above wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=2;//bottom membrane
      //copy protein pattern from bottom membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][c->CellMinJ];
      Aux1PrePat[i][j]=Aux1PrePat[i][c->CellMinJ];
      Lax3PrePat[i][j]=Lax3PrePat[i][c->CellMinJ];

      //make membrane below wall: top membrane lower daughter cell
      j=c->CellMinJ+jwall-1;//below wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=1;//top membrane
      //copy protein pattern from top membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][c->CellMaxJ];
      Aux1PrePat[i][j]=Aux1PrePat[i][c->CellMaxJ];
      Lax3PrePat[i][j]=Lax3PrePat[i][c->CellMaxJ];
    }
  //use pre-existing cell c for lowermost daughter
  //use newly generated cell cnew for uppermost daughter
  for(i=istart;i<=istop;i++)
    for(j=c->CellMinJ+jwall+1;j<=c->CellMaxJ;j++)
      {
	CellGrid[i][j]=cnew;
      }

  int poswall=c->CellMinJ+jwall;

  //printf("time %i tzposition is %i,cell type %i original id %i mini %i maxi %i minj %i maxj %i\n",time,yLRC1-(15*HMZ+15),Tc[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))],c->cellid,c->CellMinI,c->CellMaxI,c->CellMinJ,c->CellMaxJ);

  //adjust positions of corners, heights, sizes etc
  //new cell id
  cnew->CellMinI=c->CellMinI;
  cnew->CellMaxI=c->CellMaxI;
  cnew->CellWidth=c->CellWidth;
  cnew->CellMinJ=poswall+1;//above wall
  cnew->CellMaxJ=c->CellMaxJ;
  cnew->CellHeight=cnew->CellMaxJ-cnew->CellMinJ+1;
  cnew->CellSize=cnew->CellWidth*cnew->CellHeight;
  cnew->InvCellSize=1.0/cnew->CellSize;

  //printf("upper cell minj %i max j%i\n",cnew->CellMinJ,cnew->CellMaxJ);

  //cell inheriting id from mother cell
  c->CellMaxJ=poswall-1;//below wall
  c->CellHeight=c->CellMaxJ-c->CellMinJ+1;
  c->CellSize=c->CellWidth*c->CellHeight;
  c->InvCellSize=1.0/c->CellSize;


  cnew->ContCellHeight=cnew->CellHeight;//
  c->ContCellHeight=c->CellHeight;//
  //printf("lower cell minj %i max j%i\n",c->CellMinJ,c->CellMaxJ);
  //#endif

  //bookkeeping division times and numbers
  c->TimePrevGrowth=time;
  cnew->TimePrevGrowth=c->TimePrevGrowth;
  c->CellDivNr=c->CellDivNr+1;
  cnew->CellDivNr=c->CellDivNr;
#ifndef DEBUGMODE
  if(c->CellMinJ==(yLRCstraight+1))//lowermost meristem cell
    c->CellDivNr=0;
#endif
  c->CellTimeLastDiv=time;
  cnew->CellTimeLastDiv=c->CellTimeLastDiv;

  //assign division and elongation rates
#ifdef NONSTOCHASTICDIVISION
  divtime1=avgdivtime;
  divtime2=avgdivtime;
#endif
#ifdef FULLYSTOCHASTICDIVISION
  divtime1=avgdivtime*(0.95+0.1*rand()/RAND_MAX);//uniform();
  divtime2=avgdivtime*(0.95+0.1*rand()/RAND_MAX);//uniform();
#endif
#ifdef PERROWSTOCHASTICDIVISION
  int meanj1=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
  int rownr1=meanj1%1000;
  int meanj2=(int)(0.5*(cnew->CellMinJ+cnew->CellMaxJ));
  int rownr2=meanj2%1000;
  if(randnrs[rownr1]<0.333)
    divtime1=avgdivtime*0.95;
  else if(randnrs[rownr1]<0.666)
    divtime1=avgdivtime*1;
  else
    divtime1=avgdivtime*1.05;
  if(randnrs[rownr2]<0.333)
    divtime2=avgdivtime*0.95;
  else if(randnrs[rownr2]<0.666)
    divtime2=avgdivtime*1;
  else
    divtime1=avgdivtime*1.05;
  //divtime1=avgdivtime*(0.95+0.1*randnrs[rownr1]);
  //divtime2=avgdivtime*(0.95+0.1*randnrs[rownr2]);
#endif
#ifdef CELLCYCLEGRADIENT
  int  meanj1=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
  int modi1=meanj1%3;
  if(modi1==0)
    divtime1=avgdivtime*0.95;
  else if(modi1==1)
    divtime1=avgdivtime*1;
  else
    divtime1=avgdivtime*1.05;
  int meanj2=(int)(0.5*(cnew->CellMinJ+cnew->CellMaxJ));
  int modi2=meanj2%3;
  if(modi2==0)
    divtime2=avgdivtime*0.95;
  else if(modi2==1)
    divtime2=avgdivtime*1;
  else
    divtime2=avgdivtime*1.05;

  /*
  divtime1=avgdivtime*(0.95+0.05*(c->CellMinJ%3));
  divtime2=avgdivtime*(0.95+0.05*(cnew->CellMinJ%3));
  */
#endif



#ifndef DEBUGMODE

#ifdef STEMCELL
  if(Tct[c->CellMinI][c->CellMinJ]!=1)//non-lrc
    {
      if(c->CellMinJ==yLRCstraight+1)//lowermost cell
	{
	  //printf("stem cells found!\n");

	  divtime1=1.35*divtime1;
	  divtime2=1.*divtime2;

	  cnew->Ra=c->Ra;
	  c->Ra=1.;

	  /*
	  if(c->CellMinI<xm)
	    c->Ra=1.3;
	  else
	    c->Ra=1.;
	  */

	  //c->Ra=1+0.30*(-0.5+uniform());

	  //Note that this randomness is currently only applied in
	  //Aux1 expression of pericycle cells in the EZ, so you will
	  //only see it once cells that have newly arisen through division
	  //arrive there
	}
    }
#endif
#endif
  cnew->Ra=c->Ra;//


  /*
  if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==8)
    {
      double random= (1-0.2*(-0.5+uniform()));
      c->Aux1= random*c->Aux1;
      cnew->Aux1= random*cnew->Aux1;
    }
  */


  sumdiv=0;
#ifndef DEBUGMODE
  for(i=HMZ;i<=2*HMZ;i++)
#else
  for(i=M1;i<=2*M1;i++)
#endif
    sumdiv+=(1./((double)i));
  //printf("sumdiv %f i at end %i\n",sumdiv,i);
#ifdef OLDGROWTH
  divrate=((double)sumdiv)/divtime1;
#endif
#ifdef NEWGROWTH
  divrate=log((2*(double)HMZ+1)/(double)HMZ)/divtime1;
#endif
  c->Rdiv=divrate;
#ifdef OLDGROWTH
  divrate=((double)sumdiv)/divtime2;
#endif
#ifdef NEWGROWTH
  divrate=log((2*(double)HMZ+1)/(double)HMZ)/divtime2;
#endif
  cnew->Rdiv=divrate;

#ifdef VASCDIFFERENT
   //LARGER & FASTER VASCULATURE
  if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
    {
      divtime2=divtime2/1.5;
      divtime1=divtime1/1.5;

      for(i=1.5*HMZ;i<=1.5*2*HMZ;i++)
	sumdiv+=(1./((double)i));
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime1;
#endif
#ifdef NEWGROWTH
      divrate=log((1.5*2*(double)HMZ+1)/(double)1.5*HMZ)/divtime1;
#endif
      c->Rdiv=divrate;
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime2;
#endif
#ifdef NEWGROWTH
      divrate=log((1.5*2*(double)HMZ+1)/(double)1.5*HMZ)/divtime2;
#endif
      cnew->Rdiv=divrate;
    }
#endif
#ifdef VASCDIFFONLYRATE
  if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
    {
      divtime2=divtime2/1.5;
      divtime1=divtime1/1.5;

      for(i=HMZ;i<=2*HMZ;i++)
	sumdiv+=(1./((double)i));
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime1;
#endif
#ifdef NEWGROWTH
      divrate=log((2*(double)HMZ+1)/(double)HMZ)/divtime1;
#endif
      c->Rdiv=divrate;
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime2;
#endif
#ifdef NEWGROWTH
      divrate=log((2*(double)HMZ+1)/(double)HMZ)/divtime2;
#endif
      cnew->Rdiv=divrate;
    }
#endif
#ifdef VASCDIFFONLYSIZE
  //if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
  if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=9 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc 
    {
       for(i=1.5*HMZ;i<=1.5*2*HMZ;i++)
	sumdiv+=(1./((double)i));
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime1;
#endif
#ifdef NEWGROWTH
      divrate=log((1.5*2*(double)HMZ+1)/(double)1.5*HMZ)/divtime1;
#endif
      c->Rdiv=divrate;
#ifdef OLDGROWTH
      divrate=((double)sumdiv)/divtime2;
#endif
#ifdef NEWGROWTH
      divrate=log((1.5*2*(double)HMZ+1)/(double)1.5*HMZ)/divtime2;
#endif
      cnew->Rdiv=divrate;
    }
#endif
  //if everybody both faster and bigger
  /*
  divtime2=divtime2/1.5;
  divtime1=divtime1/1.5;
  
  for(i=1.5*HMZ;i<=1.5*2*HMZ;i++)
    sumdiv+=(1./((double)i));
  
  divrate=((double)sumdiv)/divtime1;
  c->Rdiv=divrate;
  divrate=((double)sumdiv)/divtime2;
  cnew->Rdiv=divrate;
  */ 


  elongtime1=avgelongtime;
  elongtime2=avgelongtime;
  sumelong=0;
#ifndef DEBUGMODE
  for(i=((int)2*HMZ);i<=Hmax;i++)
#else
  for(i=2*M1;i<=Hmax;i++)
#endif
#ifndef COMPENSATESIZEDIFFERENCESEZ
    sumelong+=(1./((double)i));
#else
  sumelong+=(1./((double)i-((((double)i/(double)HMZ)-1.)*1.5)));
  //sumelong+=(1./((double)i-((((double)i/(double)HMZ)-1.)*1.25)));
#endif
  //printf("sumelong %f i at end %i\n",sumelong,i);
#ifdef OLDGROWTH
  elongrate=((double)sumelong)/elongtime1;
#endif
#ifdef NEWGROWTH
#ifndef STAGGER
  elongrate=log(((double)Hmax)/(double)(2*HMZ))/elongtime1;
#else
  elongrate=log(((double)Hmax)/(double)(1.5*2*HMZ))/elongtime1;
#endif
#endif
  c->Relong=elongrate;
#ifdef OLDGROWTH
  elongrate=((double)sumelong)/elongtime2;
#endif
#ifdef NEWGROWTH
  elongrate=log(((double)Hmax)/(double)(2*HMZ))/elongtime2;
#endif
  cnew->Relong=elongrate;

  //printf("cells after div %i %i TimePrevGrowth %i %i timetogrowth %f %f growthrate %f %f elongrate %f %f height %i %i\n",c->cellid,cnew->cellid,c->TimePrevGrowth,cnew->TimePrevGrowth,1.0/(c->Rdiv*c->CellHeight),1.0/(cnew->Rdiv*cnew->CellHeight),c->Rdiv,cnew->Rdiv,c->Relong,cnew->Relong,c->CellHeight,cnew->CellHeight);

  //printf("%f %f %f %f %f %f\n",divtime1,divtime2,elongtime1,elongtime2,sumdiv,sumelong);
  //exit(1);

  //copy other properties
  cnew->CopyCell(c);

  //insert new cell at end of cell list
  (*CellList).push_back(cnew);
  //#endif
}


#ifdef OLDGROWTH
void Tissue::Growth(int time)
{
  iter it;
  BCell *c;
  int i,j,q;
  int H;
  double P;
  int meani,mini,maxi,meanj;
  int tgrowth;
  static int first_time=0;


  //Rdiv: intrinsic cellular growth & division rate
  //Relong: intrinsic cellular elongation rate
  //Rgrow: actual current growth rate of cell based on intrinsic rates, where it is
  //(division or elongation zone), auxin/nearness QC etc
  //normally determined if cell arises after division
  //need to be set for cells we start simulations with

#ifndef RESTART
  double divtime;
  double elongtime;
  double sumdiv;
  double divrate;
  double sumelong;
  double elongrate;
  double steptime;
  double R1;

  if(first_time==0)
    {

      if(CellList!=NULL)
	{
	  it=CellList->begin();
	  while(it!=CellList->end())
	    {
	      c=(*it);
	      divtime=avgdivtime;
	      sumdiv=0;
#ifndef DEBUGMODE
	      for(i=HMZ;i<=2*HMZ;i++)
#else
	      for(i=M1;i<=2*M1;i++)
#endif
		sumdiv+=(1./((double)i));
#ifdef NONSTOCHASTICDIVISION
	      divrate=sumdiv/divtime;
#endif
#ifdef FULLYSTOCHASTICDIVISION
	      divrate=sumdiv/(divtime*(0.95+0.1*rand()/RAND_MAX));//uniform());
#endif
#ifdef PERROWSTOCHASTICDIVISION
	      int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	      int rownr=meanj%1000;
	      if(randnrs[rownr]<0.333)
		divrate=sumdiv/(divtime*0.95);
	      else if(randnrs[rownr]<0.666)
		divrate=sumdiv/(divtime*1);
	      else
		divrate=sumdiv/(divtime*1.05);
	      //divrate=sumdiv/(divtime*(0.95+0.1*randnrs[rownr]));
#endif
#ifdef CELLCYCLEGRADIENT
	      int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	      int modi=meanj%3;
	      if(modi==0)
		divrate=sumdiv/(divtime*0.95);
	      else if(modi==1)
		divrate=sumdiv/(divtime*1);
	      else
		divrate=sumdiv/(divtime*1.05);	

	      divrate=sumdiv/(divtime*(0.95+0.05*(c->CellMinJ%3)));
#endif
	      c->Rdiv=divrate;

#ifdef VASCDIFFERENT
	      //LARGER VASCULATURE
	      if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 &&
		 Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		{
		  divtime=avgdivtime/1.5;

		  for(i=1.5*HMZ;i<=1.5*2*HMZ;i++)
		    sumdiv+=(1./((double)i));

		  divrate=((double)sumdiv)/divtime;
		  c->Rdiv=divrate;
		}
#endif
#ifdef VASCDIFFONLYRATE
	      if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 &&
		 Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		{
		  divtime=avgdivtime/1.5;

		  for(i=HMZ;i<=2*HMZ;i++)
		    sumdiv+=(1./((double)i));

		  divrate=((double)sumdiv)/divtime;
		  c->Rdiv=divrate;
		}
#endif
#ifdef VASCDIFFONLYSIZE
	      //if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
	       if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=9 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc	
		{
		  for(i=1.5*HMZ;i<=1.5*2*HMZ;i++)
		    sumdiv+=(1./((double)i));
		  
		  divrate=((double)sumdiv)/divtime;
		  c->Rdiv=divrate;
    }
#endif

	      sumelong=0;
	      elongtime=avgelongtime;
#ifndef DEBUGMODE
	      for(i=((int)2*HMZ);i<=Hmax;i++)
#else
		for(i=2*M1;i<=Hmax;i++)
#endif
#ifndef COMPENSATESIZEDIFFERENCESEZ
		  sumelong+=(1./((double)i));
#else
	      sumelong+=(1./((double)i-((((double)i/(double)HMZ)-1.)*1.5)));
	      //sumelong+=(1./((double)i-((((double)i/(double)HMZ)-1.)*1.25)));
#endif
	      elongrate=sumelong/elongtime;
	      c->Relong=elongrate;

	      it++;
	    }
	}
      first_time=1;
    }
#endif



   if(CellList!=NULL)
     {
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   c->JustChecked=0;
	   it++;
	 }
     }

   int celltype;
#ifdef SYNCEDEZENTRY
   BCell *cleft;
   BCell *cright;

   int goodtogo[m];
   int iepi=disttoepi+1;
   int icort=disttoepi+totsizeepi+1;
   int iendo=disttoepi+totsizeepi+totsizecort+1;
   int iperi=disttoepi+totsizeepi+totsizecort+totsizeendo+1;
   int ivasc=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+1;
   int count;
   for(j=0;j<m;j++)
     goodtogo[j]=-1;
   for(j=yLRC1;j<m;j++)
     {
       count=0;

       cleft=CellGrid[iepi][j];
       if(cleft==NULL || cleft->CellMinJ>yLRC1)
	 count++;
       cleft=CellGrid[icort][j];
       if(cleft==NULL || cleft->CellMinJ>yLRC1)
	 count++;
       cleft=CellGrid[iendo][j];
       if(cleft==NULL || cleft->CellMinJ>yLRC1)
	 count++;
       cleft=CellGrid[iperi][j];
       if(cleft==NULL || cleft->CellMinJ>yLRC1)
	 count++;
       cleft=CellGrid[ivasc][j];
       if(cleft==NULL || cleft->CellMinJ>yLRC1)
	 count++;

       if(count==5)
	 goodtogo[j]=1;
     }
   if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  c=(*it);
	  c->goodtogotoEZ=1;
	  if(c->CellMinJ<yLRC1)//cell itself still in MZ
	     c->goodtogotoEZ=-1;
	  else
	    {
	      for(j=c->CellMinJ;j<=c->CellMaxJ;j++)
		{
		  if(goodtogo[j]==-1)
		    {
		      c->goodtogotoEZ=-1;
		      break;
		    }
		}
	    }
	  it++;
	}
    }
#endif




  double rdivaux,relong;
  //int celltype;
  //cell based Rgrow
  if(CellList!=NULL)
     {
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   celltype=Tct[c->CellMinI][c->CellMinJ];


#ifdef NORMALTISSUEPATTERN
	   if(c->CellMaxJ<yLRCstraight)//inside curved part rootcap: ignore growth
#endif
#ifdef  SKEWEDTISSUEPATTERN
	     if(!(c->CellMinJ>yLRCstraight))    
#endif 
	     c->Rgrow=0.0000001;
#ifndef DEBUGMODE
#ifdef PLTDEPZONATION
	   else if(c->Plts>15)//MZ and in straight part: grow & divide
#else
#ifndef SYNCEDEZENTRY
	   else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc cells
#else
	   else if((celltype>4 && c->goodtogotoEZ<1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc
#endif
#endif
#else
	   else if(c->CellMinJ<Y1*M1)
#endif
	     {
	       //if(!(celltype>=8 && celltype<=12) || c->CellMinJ>yLRCstraight+4*HMZ+4)
		 c->Rgrow=c->Rdiv;
		 //else
		 //c->Rgrow=0.0000001;
	     }
	   else// EZ and DZ expand and or differentiate
	     {
	       //EZ: expand and differentiate
	       if(c->Diff<85)//(c->CellHeight<Hmax && c->Diff<85)
		 c->Rgrow=c->Relong;
	       else//DZ: no further expansion
		 c->Rgrow=0.0000001;
	     }

	   it++;
	 }
     }

  //do the actual growth
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	   c=(*it);

	   if(c->JustChecked==0 && c->CellSize>0)//extra second statement
	     {
	       H=c->CellHeight;
	       int meani=(int)(0.5*(c->CellMinI+c->CellMaxI));
	       int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	       int celltype=Tct[meani][meanj];

#ifdef NORMALTISSUEPATTERN
	       if(c->CellMaxJ<yLRCstraight)//inside curved part rootcap: ignore growth
#endif
#ifdef  SKEWEDTISSUEPATTERN
	       if(!(c->CellMinJ>yLRCstraight))    
#endif 
		 {
		   c->CellZonationState=0;
		   c->JustChecked=1;
		   it++;
		 }
#ifndef DEBUGMODE
#ifdef PLTDEPZONATION
	       else if(c->Plts>15)//MZ growth and division
#else
#ifndef SYNCEDEZENTRY
	       else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc
#else
	       else if((celltype>4 && c->goodtogotoEZ<1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc
#endif
#endif
#else
               else if(c->CellMinJ<Y1*M1)
#endif
		 {
#ifndef DEBUGMODE
		   int TZboundary;


#ifndef SPECIALTZBOUNDARY
		   TZboundary=yLRC1-(15*HMZ+15);
#else
		   {
		     if(Tct[c->CellMinI][c->CellMinJ]>8 && Tct[c->CellMinI][c->CellMinJ]<12)//vasc
		       TZboundary=yLRC1-(20*HMZ+20);
		     else if(Tct[c->CellMinI][c->CellMinJ]==8)//peri
		       TZboundary=yLRC1-(8*HMZ+8);
		     else
		       TZboundary=yLRC1-(15*HMZ+15);
		   }
#endif


		   if(c->CellMinJ<TZboundary)
		     c->CellZonationState=30;
		   else
		     c->CellZonationState=50;

		   int Hdiv2;
		   double relpos;
		   double generalrelpos;
		   double generalTZboundary=yLRC1-(15*HMZ-15);

		   relpos=(double)(c->CellMinJ-yLRCstraight)/(double)(TZboundary-yLRCstraight);
		   generalrelpos=(double)(c->CellMinJ-yLRCstraight)/(double)(generalTZboundary-yLRCstraight);

#ifdef OLDDIVSIZEGRADIENT
		  //increase div size same for diff tissues
		   if(generalrelpos<0.5)
		     Hdiv2=Hdiv;
		   else if(generalrelpos<0.625)
		     Hdiv2=Hdiv+1;
		   else if(generalrelpos<0.75)
		     Hdiv2=Hdiv+2;
		   else if(generalrelpos<0.875)
		     Hdiv2=Hdiv+3;
		   else //if(generalrelpos<1)
		     Hdiv2=Hdiv+4;
		   //else if(relpos>=1)
		   //Hdiv2=100000; 
		   
		   if(relpos>1)//stop division according to own TZ boundary
		     Hdiv2=100000;
#endif
#ifdef CORRECTEDDIVSIZEGRADIENT
		   //increase div size same for diff tissues
		   if(generalrelpos<0.65)
		     Hdiv2=Hdiv;
		   else if(generalrelpos<0.8)
		     Hdiv2=Hdiv+2;
		   else
		     Hdiv2=Hdiv+4;

		   if(relpos>1)//stop division according to own TZ boundary
		     Hdiv2=100000;
#else
		   if(relpos<1)
		     Hdiv2=Hdiv;
		   else
		     Hdiv2=100000;
#endif
#else
		   int Hdiv2=Hdiv;
#endif


#ifdef VASCDIFFERENT

		   //LARGER VASCULATURE
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		     Hdiv2*=1.5;
#endif
#ifdef VASCDIFFERENTONLYSIZE

		   //LARGER VASCULATURE
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		     Hdiv2*=1.5;
#endif



		   //Hdiv2*=1.5;


		   //please note that this can give rise to strange artifacts
		   //by putting Hdiv2 high we enable it to keep growing
		   //cytoplasmatically while not dividing
		   //normally the actual growth occurring will be limited
		   //as the cell will be pushed out by cells below it and
		   //then enter EZ and DZ where there are length boundaries
		   //however if growth below it is abnormal or ceases this
		   //cell can grow extremely big and subsequently become
		   //culled


		   c->JustChecked=1;

#ifndef GROWTHDIVSIMANDSIZECOMP
		   if(H>Hdiv2)
		     {
		       //if(celltype>4)
			 {
			   DivideCell(c,time);
			   it=CellList->begin();
			 }
			 //else
			 //it++;
		     }
		   else
		     {
		       //if(celltype>4)
			 {
			   tgrowth=1./(c->Rgrow*c->CellHeight);
			   if(time>=c->TimePrevGrowth+tgrowth)
			     {
			       GrowCell(c,time);
			   it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling
			     }
			 }
			 //else
			 //it++;
		     }
#else
		   tgrowth=1./(c->Rgrow*(c->CellHeight+1));//!!
		   if(time>=c->TimePrevGrowth+tgrowth)
		     {
		       GrowCell(c,time);
		       it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling
		     }
		   if(H>Hdiv2)
		     {
		       DivideCell(c,time);
		       it=CellList->begin();
		     }
#endif


		 }
	       else//EZ or  DZ
		 {
		   c->JustChecked=1;
		   //differentiation starts in EZ, proceeds in DZ
		   if((int)1000*c->Diff==0)//only at start of diff
		     c->Diff+=0.001;
		   else
		     c->Diff+=tstep*speedup2*(diffrate-decaydiffrate*c->Diff);

		   if(c->Diff<85)//not yet into terminal differentiation, approximately 6.94 hrs
		     {
		       c->CellZonationState=60;

		       //if(H<Hmax)//still allowed to expand
		       {
			 double auxindep=1;
			 //if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==8)
			 //auxindep=0.7+0.5*c->MeanCells*c->MeanCells/(c->MeanCells*c->MeanCells+80*80);
			 tgrowth=1./(auxindep*c->Rgrow*c->CellHeight);
#ifdef COMPENSATESIZEDIFFERENCESEZ
			 tgrowth=1./(c->Rgrow*(c->CellHeight-(((double)c->CellHeight)/((double)HMZ)-1)*1.5));
#endif
			 if(time>=c->TimePrevGrowth+tgrowth)
			   {
			     GrowCell(c,time);
			     it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling
			   }
			 //should there be an else it++
			 else
			   it++;
		       }
		       //should there be an else here in case if if(H<Hmax)
		       //and if not thise one should be outcommented?
		       //else//11 may
		       //it++;//11 may
		     }
		   else//terminal diff so DZ
		     {
		       c->CellZonationState=100;
		       it++;
		     }
		 }
	     }//cell not yet checked
	   else
	     it++;
	}//not yet at end cell list

    }//non-zero cell list

  //printf("time %i nrcells %i maxidnr %i\n",time,actcells,maxidnr);
}

#endif





#ifdef NEWGROWTH
void Tissue::Growth(int time)
{
  iter it;
  BCell *c;
  int i,j,q;
  int H;
  double P;
  int meani,mini,maxi,meanj;
  int tgrowth;
  static int first_time=0;


  //Rdiv: intrinsic cellular growth & division rate
  //Relong: intrinsic cellular elongation rate
  //Rgrow: actual current growth rate of cell based on intrinsic rates, where it is
  //(division or elongation zone), auxin/nearness QC etc
  //normally determined if cell arises after division
  //need to be set for cells we start simulations with

#ifndef RESTART
  double divtime;
  double elongtime;
  double sumdiv;
  double divrate;
  double sumelong;
  double elongrate;
  double steptime;
  double R1;

  if(first_time==0)
    {

      if(CellList!=NULL)
	{
	  it=CellList->begin();
	  while(it!=CellList->end())
	    {
	      c=(*it);
	      divtime=avgdivtime;
	      sumdiv=0;

	      for(i=HMZ;i<=2*HMZ;i++)
		sumdiv+=(1./((double)i));

	      divrate=sumdiv/divtime;

#ifdef CELLCYCLEGRADIENT
	      int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	      int modi=meanj%3;
	      if(modi==0)
		divrate=sumdiv/(divtime*0.95);
	      else if(modi==1)
		divrate=sumdiv/(divtime*1);
	      else
		divrate=sumdiv/(divtime*1.05);	

	      divrate=sumdiv/(divtime*(0.95+0.05*(c->CellMinJ%3)));
#endif
	      c->Rdiv=divrate;


	      sumelong=0;
	      elongtime=avgelongtime;
	      for(i=((int)2*HMZ);i<=Hmax;i++)
		  sumelong+=(1./((double)i));
	      elongrate=sumelong/elongtime;
	      c->Relong=elongrate;

	      it++;
	    }
	}
      first_time=1;
    }
#endif



   if(CellList!=NULL)
     {
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   c->JustChecked=0;
	   it++;
	 }
     }

   int celltype;

   double rdivaux,relong;
  //int celltype;
  //cell based Rgrow
  if(CellList!=NULL)
     {
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   celltype=Tct[c->CellMinI][c->CellMinJ];


#ifdef NORMALTISSUEPATTERN
	   if(c->CellMaxJ<yLRCstraight)//inside curved part rootcap: ignore growth
#endif
#ifdef  SKEWEDTISSUEPATTERN
	     if(!(c->CellMinJ>yLRCstraight))    
#endif 
	     c->Rgrow=0.0000001;
#ifdef PLTDEPZONATION
	   else if(c->Plts>15)//MZ and in straight part: grow & divide
#else
	   else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc cells
#endif
	     {
	       //if(!(celltype>=8 && celltype<=12) || c->CellMinJ>yLRCstraight+4*HMZ+4)
		 c->Rgrow=c->Rdiv;
		 //else
		 //c->Rgrow=0.0000001;
	     }
	   else// EZ and DZ expand and or differentiate
	     {
	       //EZ: expand and differentiate
	       if(c->Diff<85)//(c->CellHeight<Hmax && c->Diff<85)
		 c->Rgrow=c->Relong;
	       else//DZ: no further expansion
		 c->Rgrow=0.0000001;
	     }

	   it++;
	 }
     }

  
  //do the actual growth
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	   c=(*it);

	   if(c->JustChecked==0 && c->CellSize>0)//extra second statement
	     {
	       H=c->CellHeight;
	       int meani=(int)(0.5*(c->CellMinI+c->CellMaxI));
	       int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	       int celltype=Tct[meani][meanj];

#ifdef NORMALTISSUEPATTERN
	       if(c->CellMaxJ<yLRCstraight)//inside curved part rootcap: ignore growth
#endif
#ifdef  SKEWEDTISSUEPATTERN
	       if(!(c->CellMinJ>yLRCstraight))    
#endif 
		 {
		   c->CellZonationState=0;
		   c->JustChecked=1;
		   it++;
		 }
#ifdef PLTDEPZONATION
	       else if(c->Plts>15)//MZ growth and division
#else
	       else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc
#endif
		 {
		   int TZboundary;
#ifndef SPECIALTZBOUNDARY
		   TZboundary=yLRC1-(15*HMZ+15);
#else
		   {
		     if(Tct[c->CellMinI][c->CellMinJ]>8 && Tct[c->CellMinI][c->CellMinJ]<12)//vasc
		       TZboundary=yLRC1-(20*HMZ+20);
		     else if(Tct[c->CellMinI][c->CellMinJ]==8)//peri
		       TZboundary=yLRC1-(8*HMZ+8);
		     else
		       TZboundary=yLRC1-(15*HMZ+15);
		   }
#endif
		   if(c->CellMinJ<TZboundary)
		     c->CellZonationState=30;
		   else
		     c->CellZonationState=50;

		   int Hdiv2;
		   double relpos;
		   double generalrelpos;
		   double generalTZboundary=yLRC1-(15*HMZ-15);

		   relpos=(double)(c->CellMinJ-yLRCstraight)/(double)(TZboundary-yLRCstraight);
		   generalrelpos=(double)(c->CellMinJ-yLRCstraight)/(double)(generalTZboundary-yLRCstraight);

		   if(relpos<1)
		     Hdiv2=Hdiv;
		   else
		     Hdiv2=100000;
#ifdef STAGGER
		   	   
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=9 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc:6
		     Hdiv2*=1.5;
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==5 || Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==7) //epi, cendo:5
		     Hdiv2*=1.3;
		   //if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==6)//cort: 3
		   //Hdiv2*=0.8;
		   //peri en cort 4
#endif		   


#ifdef VASCDIFFERENT

		   //LARGER VASCULATURE
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		     Hdiv2*=1.5;
#endif
#ifdef VASCDIFFERENTONLYSIZE

		   //LARGER VASCULATURE
		   if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]>=8 && Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]<12)//vasc
		     Hdiv2*=1.5;
#endif

		   c->JustChecked=1;

		   c->ContCellHeight+=tstep*speedup2*(c->Rgrow*(c->ContCellHeight+1.0));//added .0
		  
		   if((int)(c->ContCellHeight-c->CellHeight)>=1)//added (int)
		     {
		      
		       GrowCell(c,time);
		       it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling
		       //H=c->CellHeight;
		     }
		   if(H>Hdiv2)/////???????je gaat naar begin cell list dus als goed is H geupdate? maar justchecked op 1,
		     {
		       DivideCell(c,time);
		       it=CellList->begin();
		     }	
		   //else
		   //it++;
		 }
	       else//EZ or  DZ
		 {
		   c->JustChecked=1;
		   //differentiation starts in EZ, proceeds in DZ
		   if((int)1000*c->Diff==0)//only at start of diff
		     c->Diff+=0.001;
		   else
		     c->Diff+=tstep*speedup2*(diffrate-decaydiffrate*c->Diff);

		   if(c->Diff<85)//not yet into terminal differentiation, approximately 6.94 hrs
		     {
		       c->CellZonationState=60;

		       //if(H<Hmax)//still allowed to expand
		       {
			 double auxindep=1;
			
			 c->ContCellHeight+=tstep*speedup2*(c->Rgrow*(c->ContCellHeight+1));
			 if((c->ContCellHeight-c->CellHeight)>=1)
			   {
			     GrowCell(c,time);
			     it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling
			   }
			 else
			   it++;
		       }
		       //should there be an else here in case if if(H<Hmax)
		       //and if not thise one should be outcommented?
		       //else//11 may
		       //it++;//11 may
		     }
		   else//terminal diff so DZ
		     {
		       c->CellZonationState=100;
		       it++;
		     }
		 }
	     }//cell not yet checked
	   else
	     it++;
	}//not yet at end cell list

    }//non-zero cell list

  //printf("time %i nrcells %i maxidnr %i\n",time,actcells,maxidnr);
}

#endif




void Tissue::CullOverflowCells(int meani)
{
  int ii,jj;
  BCell *c;
  iter it, newit;
  if(Tc[meani][m-2]==1 || Tc[meani][m-2]==2)//culling needed!
    {
  c=CellGrid[meani][m-2];

  //printf("removing normal cell %i\n",c->cellid);

      //always remove:
      //wall atop of to be removed cell
      //cell itself
      //conditional remove:
      //left and right walls
      //only if no other cell there
      for(ii=(c->CellMinI-1);ii<=(c->CellMaxI+1);ii++)
	for(jj=c->CellMinJ;jj<=(c->CellMaxJ+1);jj++)
	  {
	    if(ii==(c->CellMinI-1)  &&
	       (CellGrid[ii-1][jj]!=NULL || CellGrid[ii-1][jj-1]!=NULL))
	      ;
	    else if(ii==(c->CellMaxI+1) &&
		    (CellGrid[ii+1][jj]!=NULL || CellGrid[ii+1][jj-1]!=NULL))
	      ;
	    else
	      {
		//put all to non-tissue status
		Tc[ii][jj]=3;
		Tct[ii][jj]=0;
		Tmt[ii][jj]=0;
		PinPrePat[ii][jj]=0;
		//more to come
		CellGrid[ii][jj]=NULL;
		Caux1[ii][jj]=-10;//0;
	      }
	  }
      actcells--;//adjust cell number

      //printf("erasing cell %i now still %i cells\n",c->cellid,actcells);
      it=CellList->begin();//find and remove cell from list
      while(it!=CellList->end())
	{
	  if((*it)==c)
	    {
	      //printf("doing the actual erasing \n");
	      //newit=it;
	      //newit++;
	      delete (c);//(*it);
	      it=CellList->erase(it);
	      //it=newit;
	      it=CellList->end();//when found break from loop
	    }
	  it++;
	}
    }
}


void Tissue::CullLateralRootcap(int side)
{
#ifndef DEBUGMODE
  int i,j,k,ii,jj,iii,jjj;
  int q,qq;
  int meanJ;
  BCell *c;
  BCell *cc;
  iter it;

  if(side==0)//left lateral root cap
    i=disttolrc+1;
  else//right lateral root cap
    i=n-1-(disttolrc+1);

  for(j=m-1;j>=0;j--)
    {
      if(Tc[i][j]==2)//uppermost lateral root cap found
	{
	  c=CellGrid[i][j];

	  //printf("removing lateral root cap cell %i\n",c->cellid);
#ifdef PLTDEPZONATION
	  //find neighbouring epidermal cell
	  meanJ=0.5*(c->CellMinJ+c->CellMaxJ[q]);
	  ii=i;
	  if(side==0)//left root cap so epidermis to the right
	    {
	      while(Cell[ii][j]==Cell[i][j] || Cell[ii][j]==-1)//buggy if in horizontal wall
		ii++;
	    }
	  else//right root cap so epidermis to the left
	    {
	      while(Cell[ii][j]==Cell[i][j] || Cell[ii][j]==-1)//buggy if in horizontal wall
		ii--;
	    }

	  cc=CellGrid[ii][j];
	  if(cc->Plts<15)//no longer MZ
#else
	  if(c->CellMinJ>yLRC1)
#endif
	    {
	      //removing cell itself+ wall above
	      for(ii=c->CellMinI;ii<=c->CellMaxI;ii++)
		for(jj=c->CellMinJ;jj<=(c->CellMaxJ+1);jj++)
		  {
		    //put all to non-tissue status
		    Tc[ii][jj]=3;
		    Tct[ii][jj]=0;
		    Tmt[ii][jj]=0;
		    PinPrePat[ii][jj]=0;
		    //more to come
		    CellGrid[ii][jj]=NULL;
		    Caux1[ii][jj]=-10;//0;
		  }

	      if(side==0)
		iii=c->CellMinI-1;//wall left can go for left lrc
	      else
		iii=c->CellMaxI+1;//wall right can go for right lrc
	      for(jjj=c->CellMinJ;jjj<=(c->CellMaxJ+1);jjj++)
		{
		  Tc[iii][jjj]=3;
		  Tct[iii][jjj]=0;
		  Tmt[iii][jjj]=0;
		  CellGrid[iii][jjj]=NULL;
		  Caux1[iii][jjj]=-10;//0;
		}

	      actcells--;//adjust cell number
	      it=CellList->begin();//find and remove cell from list
	      while(it!=CellList->end())
		{
		  if((*it)==c)
		    {
		      delete (*it);
		      it=CellList->erase(it);
		      it=CellList->end();//when found break from loop
		    }
		  it++;
		}
	    }
	}
    }
#endif
}
