#include "Tissue.hh"

#include "BCell.hh"


void Tissue::GrowCell(BCell *c,int time)
{
  int oldh;
  int growthtype;
  int istart,istop;
  int jstart,jstop;
  int i,j;
  BCell *cc;

  oldh=c->CellHeight;

  //shift all that lies above, and upper membrane of cell itself upwards
  istart=c->CellMinI-1;
  istop=c->CellMaxI+1;
  jstart=c->CellMaxJ;//top membrane
  //find uppermost j position within this cell file
  i=c->CellMinI+1;
  j=m-1;
  while(Tc[i][j]==3)
    j--;
  jstop=j+1;//j;//j+1;//ERROR perhaps should be j?

  cc=NULL;
  for(j=jstop;j>=jstart;j--)//j>=jstart so including topmembrane cell q
    for(i=istart;i<=istop;i++)
      {
	//if it is left cell, if the point is on the right wall
	//and there is a cell right of that right wall: do not
	//do anything and leave it to the right cell
	//similarly if it is a right cell, if the point is on the left 
	//wall and there is a cell left of that left wall: do not
	//do anything and leave it to the left cell
	if((c->CellMaxI<(n/2-1) && i==istop) && Tc[istop][j+1]==0)
	  ;
	else if((c->CellMaxI>(n/2-1) && i==istart) && Tc[istart][j+1]==0)
	  ;
	else
	  {
	    //Tc: shift cell, membrane, wall locations; 
	    //Tct, Tmt: shift celltypes, membrane types;
	    //CellGrid: shift were cells are at grid
	    Tc[i][j+1]=Tc[i][j];
	    Tct[i][j+1]=Tct[i][j];
	    Tmt[i][j+1]=Tmt[i][j];
	    CellGrid[i][j+1]=CellGrid[i][j];
	    //shift Pin/Lax3/Lax2/Aux1
	    PinPrePat[i][j+1]=PinPrePat[i][j];
	    Aux1PrePat[i][j+1]=Aux1PrePat[i][j];
	    Lax3PrePat[i][j+1]=Lax3PrePat[i][j];
	    //shift auxin levels
	    Caux1[i][j+1]=Caux1[i][j];
	    
	    
	    //not in old program, to be more efficient
	    //instead of finding cellcorners and cellsizes for all cells again
	    if(CellGrid[i][j+1]!=NULL)//cell there
	      {
		if(CellGrid[i][j+1]!=cc)//new cell there
		  {
		    cc=CellGrid[i][j+1];//assign as current cell
		    if(cc!=c)//GAAT DIT GOED???
		      {
			cc->CellMinJ++;//cell is being shifted so j
			cc->CellMaxJ++;//positions should be done+1
			//all else stays same: i positions, width
			//height, sizes
		      }
		  }
	      }
	  }
      }
  
  //add extra row to cell, mind membranes, PINs etc
  j=c->CellMaxJ-1; 
  for(i=istart;i<=istop;i++)
    {
      //newly aroson gap is now filled with extra row by copying below row
      Tc[i][j+1]=Tc[i][j];
      Tct[i][j+1]=Tct[i][j];
      Tmt[i][j+1]=Tmt[i][j];
      CellGrid[i][j+1]=CellGrid[i][j];
      //correct Pin/Lax3/Lax2/Aux1 in extra lateral membrane?! (dilute??)
      PinPrePat[i][j+1]=PinPrePat[i][j];
      Aux1PrePat[i][j+1]=Aux1PrePat[i][j]; 
      Lax3PrePat[i][j+1]=Lax3PrePat[i][j];
    }
  
    
  //growing cell is only one that changes in height and hence size
  //not in old program, to be more efficient
  //instead of finding cellcorners and cellsizes for all cells again
  c->CellMaxJ++;
  c->CellHeight++;
  c->CellSize+=c->CellWidth;
  c->InvCellSize=1./c->CellSize;
  
  //DILUTE AUXIN, otherwise auxin created through growth
  for(i=istart;i<=istop;i++)
    for(j=c->CellMinJ;j<=c->CellMaxJ;j++)
      {
	Caux1[i][j]*=oldh/(oldh+1.);
      }
  //dilute concentrations of proteins if necessary
  if(c->CellZonationState<60)
    growthtype=0;//cytoplasmic 
  else
    growthtype=1;//vacuolar
  c->DiluteCell(c,growthtype,oldh);
  
  //bookkeeping of growth times
  c->TimePrevGrowth=time;
  
  
  //restrict to relevant cell file
  //so also different and more efficient then in old program
  int meani=(int)(0.5*(c->CellMinI+c->CellMaxI));
  int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
  CullOverflowCells(meani); 
#ifndef DEBUGMODE
#ifndef SMBMUTANT
  //we only need to check for the need for lateral root 
  //cap culling if growth occurs in this file!!!!
  if(Tct[meani][meanj]==1)//lrc1 so root cap culling relevant
    {
      if(meani<n/2)
	CullLateralRootcap(0);
      else
	CullLateralRootcap(1);
    }
#endif
#endif
}



void Tissue::DivideCell(BCell *c, int time)
{
  BCell *cnew;
  int i,j;
  double divtime1,divtime2;
  double sumdiv,sumelong;
  double divrate,elongrate;
  double elongtime1,elongtime2;
  int istart,istop,jwall;

 
  //first make new cell (need id and pointer)
  cnew=new BCell();
  maxidnr++;
  actcells++;
  cnew->CreateCell(maxidnr);

  //printf("cell %i will divide, created new cell with id %i, nr cells now %i\n",c->cellid,cnew->cellid,actcells);
  

  //#ifdef TESTING
  //insert wall, membranes and PINs on grid and 
  //divide grid points over two cells
  //cell c is up for a division
  istart=c->CellMinI;
  istop=c->CellMaxI;
  jwall=c->CellHeight/2;

  for(i=istart;i<=istop;i++)
    {
      //make wall:
      j=c->CellMinJ+jwall;//wall
      Tc[i][j]=0;//wall
      Tct[i][j]=20;//wall
      Tmt[i][j]=0;//wall
      CellGrid[i][j]=NULL;//wall

      //make membrane above wall: bottom membrane top daughter cell
      j=c->CellMinJ+jwall+1;//above wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=2;//bottom membrane 
      //copy protein pattern from bottom membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][c->CellMinJ];
      Aux1PrePat[i][j]=Aux1PrePat[i][c->CellMinJ];
      Lax3PrePat[i][j]=Lax3PrePat[i][c->CellMinJ];
 
      //make membrane below wall: top membrane lower daughter cell
      j=c->CellMinJ+jwall-1;//below wall
      Tc[i][j]=2;//membrane
      Tmt[i][j]=1;//top membrane
      //copy protein pattern from top membrane mother cell
      PinPrePat[i][j]=PinPrePat[i][c->CellMaxJ];
      Aux1PrePat[i][j]=Aux1PrePat[i][c->CellMaxJ];
      Lax3PrePat[i][j]=Lax3PrePat[i][c->CellMaxJ]; 
    }
  //use pre-existing cell c for lowermost daughter
  //use newly generated cell cnew for uppermost daughter
  for(i=istart;i<=istop;i++)
    for(j=c->CellMinJ+jwall+1;j<=c->CellMaxJ;j++)
      {
	CellGrid[i][j]=cnew;
      }

  int poswall=c->CellMinJ+jwall;

  //printf("time %i tzposition is %i,cell type %i original id %i mini %i maxi %i minj %i maxj %i\n",time,yLRC1-(15*HMZ+15),Tc[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))],c->cellid,c->CellMinI,c->CellMaxI,c->CellMinJ,c->CellMaxJ);

  //adjust positions of corners, heights, sizes etc
  //new cell id
  cnew->CellMinI=c->CellMinI;
  cnew->CellMaxI=c->CellMaxI;
  cnew->CellWidth=c->CellWidth;
  cnew->CellMinJ=poswall+1;//above wall
  cnew->CellMaxJ=c->CellMaxJ;
  cnew->CellHeight=cnew->CellMaxJ-cnew->CellMinJ+1;
  cnew->CellSize=cnew->CellWidth*cnew->CellHeight;
  cnew->InvCellSize=1.0/cnew->CellSize;

  //printf("upper cell minj %i max j%i\n",cnew->CellMinJ,cnew->CellMaxJ);

  //cell inheriting id from mother cell
  c->CellMaxJ=poswall-1;//below wall
  c->CellHeight=c->CellMaxJ-c->CellMinJ+1;
  c->CellSize=c->CellWidth*c->CellHeight;
  c->InvCellSize=1.0/c->CellSize;

  //printf("lower cell minj %i max j%i\n",c->CellMinJ,c->CellMaxJ);
  //#endif

  //bookkeeping division times and numbers
  c->TimePrevGrowth=time;
  cnew->TimePrevGrowth=c->TimePrevGrowth;
  c->CellDivNr=c->CellDivNr+1;
  cnew->CellDivNr=c->CellDivNr;
#ifndef DEBUGMODE
  if(c->CellMinJ==(yLRCstraight+1))//lowermost meristem cell
    c->CellDivNr=0;  
#endif
  c->CellTimeLastDiv=time;
  cnew->CellTimeLastDiv=c->CellTimeLastDiv;
 
  //assign division and elongation rates
  divtime1=avgdivtime;
  divtime2=avgdivtime;
#ifndef DEBUGMODE
  if(Tct[c->CellMinI][c->CellMinJ]!=1)//non-lrc
    {
      if(c->CellMinJ==yLRCstraight+1)//lowermost cell
	{
	  divtime2=1.35*divtime2;
	  divtime1=1.*divtime1;

	  cnew->Ra=c->Ra;
	  c->Ra=1+0.40*(-0.5+uniform());
	}
    }
#endif
  cnew->Ra=c->Ra;//


  /*
  if(Tct[(int)(0.5*(c->CellMinI+c->CellMaxI))][(int)(0.5*(c->CellMinJ+c->CellMaxJ))]==8)
    {
      double random= (1-0.2*(-0.5+uniform()));
      c->Aux1= random*c->Aux1;
      cnew->Aux1= random*cnew->Aux1;
    }
  */
  
  sumdiv=0;
#ifndef DEBUGMODE
  for(i=HMZ;i<=2*HMZ;i++)
#else
  for(i=M1;i<=2*M1;i++)
#endif
    sumdiv+=(1./((double)i));
  //printf("sumdiv %f i at end %i\n",sumdiv,i);
  divrate=((double)sumdiv)/divtime1;
  c->Rdiv=divrate;
  divrate=((double)sumdiv)/divtime2;
  cnew->Rdiv=divrate;

  elongtime1=avgelongtime;
  elongtime2=avgelongtime;
  sumelong=0;
#ifndef DEBUGMODE
  for(i=((int)2*HMZ);i<=Hmax;i++)
#else
  for(i=2*M1;i<=Hmax;i++)
#endif
    sumelong+=(1./((double)i));
  //printf("sumelong %f i at end %i\n",sumelong,i);
  elongrate=((double)sumelong)/elongtime1;
  c->Relong=elongrate;
  elongrate=((double)sumelong)/elongtime2;
  cnew->Relong=elongrate;

  //printf("cells after div %i %i TimePrevGrowth %i %i timetogrowth %f %f growthrate %f %f elongrate %f %f height %i %i\n",c->cellid,cnew->cellid,c->TimePrevGrowth,cnew->TimePrevGrowth,1.0/(c->Rdiv*c->CellHeight),1.0/(cnew->Rdiv*cnew->CellHeight),c->Rdiv,cnew->Rdiv,c->Relong,cnew->Relong,c->CellHeight,cnew->CellHeight);

  //printf("%f %f %f %f %f %f\n",divtime1,divtime2,elongtime1,elongtime2,sumdiv,sumelong);
  //exit(1);

  //copy other properties
  cnew->CopyCell(c);

  //insert new cell at end of cell list
  (*CellList).push_back(cnew);
  //#endif
}

void Tissue::Growth(int time)
{
  iter it;
  BCell *c;
  int i,j,q;
  int H;
  double P;
  int meani,mini,maxi,meanj;
  int tgrowth;
  static int first_time=0;


  //Rdiv: intrinsic cellular growth & division rate
  //Relong: intrinsic cellular elongation rate
  //Rgrow: actual current growth rate of cell based on intrinsic rates, where it is
  //(division or elongation zone), auxin/nearness QC etc
  //normally determined if cell arises after division
  //need to be set for cells we start simulations with

#ifndef RESTART
  double divtime;
  double elongtime;
  double sumdiv;
  double divrate;
  double sumelong;
  double elongrate;
  double steptime;
  double R1;
  if(first_time==0)
    {
      if(CellList!=NULL)
	{  
	  it=CellList->begin();
	  while(it!=CellList->end())
	    {
	      c=(*it);
	      divtime=avgdivtime;
	      sumdiv=0;
#ifndef DEBUGMODE
	      for(i=HMZ;i<=2*HMZ;i++)
#else
	      for(i=M1;i<=2*M1;i++)
#endif
		sumdiv+=(1./((double)i));
	      divrate=sumdiv/divtime;
	      c->Rdiv=divrate;

	      sumelong=0;
	      elongtime=avgelongtime;
#ifndef DEBUGMODE
	      for(i=((int)2*HMZ);i<=Hmax;i++)
#else
		for(i=2*M1;i<=Hmax;i++)
#endif
		sumelong+=(1./((double)i));
	      elongrate=sumelong/elongtime;
	      c->Relong=elongrate;
	      
	      it++;
	    }
	}
      first_time=1;
    }
#endif
 
   if(CellList!=NULL)
     {  
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   c->JustChecked=0;
	   it++;
	 }
     }
 
  double rdivaux,relong;
  int celltype;
  //cell based Rgrow
  if(CellList!=NULL)
     {  
       it=CellList->begin();
       while(it!=CellList->end())
	 {
	   c=(*it);
	   celltype=Tct[c->CellMinI][c->CellMinJ];

#ifndef DEBUGMODE
	   if(c->CellMaxJ<yR)//SCN: ignore growth
#else
	   if(c->CellMaxJ<0)
#endif
	     c->Rgrow=0.0000001;
#ifndef DEBUGMODE
	   else if(c->CellMaxJ<yLRCstraight)//inside curved part rootcap: ignore growth 
#else
	   else if(c->CellMaxJ<bufsize)  
#endif
	     c->Rgrow=0.0000001;
#ifndef DEBUGMODE
#ifdef PLTDEPZONATION
	   else if(c->Plts>15)//MZ and in straight part: grow & divide
#else
	   else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc cells
#endif 
#else
	   else if(c->CellMinJ<Y1*M1)  
#endif                                     
	     c->Rgrow=c->Rdiv;
	   else// EZ and DZ expand and or differentiate
	     {
	       //EZ: expand and differentiate
	       if(c->CellHeight<Hmax && c->Diff<85)
		 c->Rgrow=c->Relong;
	       else//DZ: no further expansion
		 c->Rgrow=0.0000001;
	     }

	   it++;
	 }
     }

  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	   c=(*it);
	   
	   if(c->JustChecked==0 && c->CellSize>0)//extra second statement
	     {                                                     
	       H=c->CellHeight;
	       int meani=(int)(0.5*(c->CellMinI+c->CellMaxI));
	       int meanj=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
	       int celltype=Tct[meani][meanj];
	       
#ifndef DEBUGMODE
	       if(c->CellMaxJ<yR)//SCN: ignore growth
#else
	       if(c->CellMaxJ<0)
#endif
		 {
		   c->CellZonationState=0;
		   c->JustChecked=1;
		   it++;
		 }
#ifndef DEBUGMODE
	       else if(c->CellMaxJ<yLRCstraight)//curved part MZ: ignore growth
#else
	       else if(c->CellMaxJ<bufsize)
#endif
		 {
		   c->CellZonationState=30;
		   c->JustChecked=1;
		   it++;
		 }
#ifndef DEBUGMODE
#ifdef PLTDEPZONATION
	       else if(c->Plts>15)//MZ growth and division
#else
	       else if((celltype>4 && c->CellMinJ<yLRC1) ||
		   c->CellMinJ<yLRC1-25*HMZ)//else if(c->CellMinJ<yLRC1)//longer lrc
#endif
#else
               else if(c->CellMinJ<Y1*M1)
#endif
		 {
#ifndef DEBUGMODE
		   int TZboundary;
		   TZboundary=yLRC1-(15*HMZ+15);
		   if(c->CellMinJ<TZboundary)
		     c->CellZonationState=30;
		   else
		     c->CellZonationState=50;

		   int Hdiv2;
		   double relpos;
		   relpos=(double)(c->CellMinJ-yLRCstraight)/(double)(TZboundary-yLRCstraight);
		    		      

		   if(relpos<0.5)
		     Hdiv2=Hdiv;
		   else if(relpos<0.625)
		     Hdiv2=Hdiv+1;
		   else if(relpos<0.75)
		     Hdiv2=Hdiv+2;
		   else if(relpos<0.875)
		     Hdiv2=Hdiv+3;
		   else if(relpos<1)
		     Hdiv2=Hdiv+4;
		   else if(relpos>=1)
		     Hdiv2=100000;
#else
		   int Hdiv2=Hdiv;
#endif    
		   
		   //please note that this can give rise to strange artifacts
		   //by putting Hdiv2 high we enable it to keep growing
		   //cytoplasmatically while not dividing
		   //normally the actual growth occurring will be limited
		   //as the cell will be pushed out by cells below it and
		   //then enter EZ and DZ where there are length boundaries
		   //however if growth below it is abnormal or ceases this
		   //cell can grow extremely big and subsequently become
		   //culled
		    
		   
		   c->JustChecked=1;
		   
		   
		   if(H>Hdiv2)
		     {
		       //if(celltype>4)
			 {
			   DivideCell(c,time);
			   it=CellList->begin();
			 }
			 //else
			 //it++;
		     }
		   else
		     {
		       //if(celltype>4)
			 {
			   tgrowth=1./(c->Rgrow*c->CellHeight);	
			   if(time>=c->TimePrevGrowth+tgrowth)
			     {
			       GrowCell(c,time);
			   it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling 
			     } 
			 }
			 //else
			 //it++;
		     }
		 }
	       else//EZ or  DZ
		 {
		   c->JustChecked=1;
		   //differentiation starts in EZ, proceeds in DZ
		   if((int)1000*c->Diff==0)//only at start of diff
		     c->Diff+=0.001;
		   else
		     c->Diff+=tstep*speedup2*(diffrate-decaydiffrate*c->Diff);
		   
		   if(c->Diff<85)//not yet into terminal differentiation
		     {
		       c->CellZonationState=60;
		       
		       if(H<Hmax)//still allowed to expand
			 {
			   tgrowth=1./(c->Rgrow*c->CellHeight);
			   if(time>=c->TimePrevGrowth+tgrowth)
			     {
			       GrowCell(c,time);
			       it=CellList->begin();//is dit wel nodig? geen renumbering, maar ja wel culling 
			     }
			 }//ONE BRACKET TOO EARLY?
			else
			 {
			   c->CellZonationState=100;  
			   it++;
			 } 	 
		     }
		   else
		     it++;
		 }
	     }//cell not yet checked
	   else
	     it++;
	}//not yet at end cell list

    }//non-zero cell list

  //printf("time %i nrcells %i maxidnr %i\n",time,actcells,maxidnr);
}


void Tissue::CullOverflowCells(int meani)
{
  int ii,jj;
  BCell *c;
  iter it, newit;
  if(Tc[meani][m-2]==1 || Tc[meani][m-2]==2)//culling needed!
    {
  c=CellGrid[meani][m-2];
  
  //printf("removing normal cell %i\n",c->cellid);
 
      //always remove: 
      //wall atop of to be removed cell 
      //cell itself
      //conditional remove:
      //left and right walls
      //only if no other cell there     
      for(ii=(c->CellMinI-1);ii<=(c->CellMaxI+1);ii++)
	for(jj=c->CellMinJ;jj<=(c->CellMaxJ+1);jj++)
	  {
	    if(ii==(c->CellMinI-1)  && 
	       (CellGrid[ii-1][jj]!=NULL || CellGrid[ii-1][jj-1]!=NULL))
	      ;
	    else if(ii==(c->CellMaxI+1) && 
		    (CellGrid[ii+1][jj]!=NULL || CellGrid[ii+1][jj-1]!=NULL))
	      ;
	    else
	      {
		//put all to non-tissue status
		Tc[ii][jj]=3;
		Tct[ii][jj]=0;
		Tmt[ii][jj]=0;
		PinPrePat[ii][jj]=0;
		//more to come
		CellGrid[ii][jj]=NULL;
		Caux1[ii][jj]=-10;//0;
	      } 
	  }
      actcells--;//adjust cell number

      //printf("erasing cell %i now still %i cells\n",c->cellid,actcells);
      it=CellList->begin();//find and remove cell from list
      while(it!=CellList->end())
	{
	  if((*it)==c)
	    {
	      //printf("doing the actual erasing \n");
	      //newit=it;
	      //newit++;
	      delete (c);//(*it);
	      it=CellList->erase(it);
	      //it=newit;
	      it=CellList->end();//when found break from loop
	    }
	  it++;
	}
    }     
}


void Tissue::CullLateralRootcap(int side)
{
#ifndef DEBUGMODE
  int i,j,k,ii,jj,iii,jjj;
  int q,qq;
  int meanJ;
  BCell *c;
  BCell *cc;
  iter it;
  
  if(side==0)//left lateral root cap
    i=disttolrc+1;
  else//right lateral root cap
    i=n-1-(disttolrc+1);
  
  for(j=m-1;j>=0;j--)
    {
      if(Tc[i][j]==2)//uppermost lateral root cap found
	{
	  c=CellGrid[i][j];
	  
	  //printf("removing lateral root cap cell %i\n",c->cellid);
#ifdef PLTDEPZONATION
	  //find neighbouring epidermal cell
	  meanJ=0.5*(c->CellMinJ+c->CellMaxJ[q]);
	  ii=i;
	  if(side==0)//left root cap so epidermis to the right
	    {
	      while(Cell[ii][j]==Cell[i][j] || Cell[ii][j]==-1)//buggy if in horizontal wall
		ii++;
	    }
	  else//right root cap so epidermis to the left
	    {
	      while(Cell[ii][j]==Cell[i][j] || Cell[ii][j]==-1)//buggy if in horizontal wall
		ii--;
	    }
	  
	  cc=CellGrid[ii][j];
	  if(cc->Plts<15)//no longer MZ
#else
	  if(c->CellMinJ>yLRC1)
#endif  
	    {
	      //removing cell itself+ wall above
	      for(ii=c->CellMinI;ii<=c->CellMaxI;ii++)
		for(jj=c->CellMinJ;jj<=(c->CellMaxJ+1);jj++)
		  {    
		    //put all to non-tissue status
		    Tc[ii][jj]=3;
		    Tct[ii][jj]=0;
		    Tmt[ii][jj]=0;
		    PinPrePat[ii][jj]=0;
		    //more to come
		    CellGrid[ii][jj]=NULL;
		    Caux1[ii][jj]=-10;//0;		      
		  }
	      
	      if(side==0)
		iii=c->CellMinI-1;//wall left can go for left lrc
	      else
		iii=c->CellMaxI+1;//wall right can go for right lrc	
	      for(jjj=c->CellMinJ;jjj<=(c->CellMaxJ+1);jjj++)
		{
		  Tc[iii][jjj]=3;
		  Tct[iii][jjj]=0;
		  Tmt[iii][jjj]=0;
		  CellGrid[iii][jjj]=NULL;
		  Caux1[iii][jjj]=-10;//0;
		}
	
	      actcells--;//adjust cell number
	      it=CellList->begin();//find and remove cell from list
	      while(it!=CellList->end())
		{
		  if((*it)==c)
		    {
		      delete (*it);
		      it=CellList->erase(it);
		      it=CellList->end();//when found break from loop
		    }
		  it++;
		}
	    }
	}
    }
#endif
}
