#include "Tissue.hh"

#include "BCell.hh"


void Tissue::SaveTotAuxin(int t)
{
  FILE *f;
  int i,j;
  char s[900];

  double Auxtot=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Auxtot+=Caux1[i][j];
      }

  sprintf(s,"%s/Totalauxin",dirname);
  f=fopen(s,"a");
  fprintf(f,"%i\t%f\t%f\n",t,Auxtot,Auxtot/(n*m));
  fclose(f);
}


void Tissue::TrackCells(int time)
{
#ifndef DEBUGMODE
  BCell *c;
  iter it;
  static int first_time=0;
  static int peri_i=disttoepi+totsizeepi+totsizeendo+totsizecort+1;
  static int peri_j=yLRC1-5*HMZ-5-1;
  int i;
  FILE *f;
  char fname[600];
  int cellstillexists;

  if(first_time==0)//initialisation
    {
      memid=-1;
      for(i=0;i<3000;i++)
	{
	  trackcelllist[i]=NULL;
	  trackcellidlist[i]=-1;
	}

      first_time=1;
      idcounter=0;
    }
  else//the real stuff
    {
      c=CellGrid[peri_i][peri_j];
      if(c!=NULL && c->cellid!=memid)//new cell arrived!
	{
	  memid=c->cellid;
	  idcounter++;
	  trackcelllist[idcounter]=c;//store pointer to cell
	  trackcellidlist[idcounter]=c->cellid;//store cell id

	  printf("found cell %i\n",c->cellid);
	}
      if(idcounter>=3000)
	printf("idcounter is larger then 3000, array overflow!\n");


      if(time%300==0 && idcounter>0)//each 100th timestep, starting once a first cell is found
	{
	  for(i=1;i<=min(idcounter,2999);i++)//for each cell thusfar found at end MZ
	    {
	      cellstillexists=0;//find if that cell still exists
	      it=CellList->begin();
	      while(it!=CellList->end())//loop over cell list
		{
		  c=(*it);
		  if(c->cellid==trackcellidlist[i])//cell with this id still exists
		    {
		      cellstillexists=1;
		      break;
		    }
		  it++;
		}
	      if(cellstillexists==1)//if cell still exists
		{
		  c=trackcelllist[i];//access the cell
		  sprintf(fname,"%s/TrackCell_%d",dirname,i);//writing in unique file per cell
		  f=fopen(fname,"a");
		  //write time, auxin, Aux1, diff status, cell height, cell j position,
		  fprintf(f,"%f\t%f\t%f\t%f\t%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",(time/3600.0),c->MeanCells,c->Aux1,c->PP,c->CellHeight,c->Diff,(0.5*(c->CellMinJ+c->CellMaxJ)),c->memARF,c->memARFtot,c->Pin,c->AuxinSign,c->CumAuxin);
		  fclose(f);
		}
	    }
	}
    }
#endif
}

void Tissue::SaveTemp(int t,int II,int JJ)
{
  FILE *f;					
  char s[1000];
  BCell *c;

  if(CellGrid[II][JJ]!=NULL)
    {
      c=CellGrid[II][JJ];
      sprintf(s,"%s/TemporalDynamicsPoint%i_%i",dirname,II,JJ);
      f=fopen(s,"a");
      fprintf(f,"%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",t,c->MeanCells,c->ARF,c->AA,c->BB,c->CC,c->DD,c->Plts,c->Pltsexpr,c->Diff);
      fclose(f);
    }
}

void Tissue::SaveGradCellularRes(int t)
{
  FILE *g;
  int i,ii,j;
  BCell *c;
  char s[1000];

  for(int k=0;k<8;k++)
    {
      if(k==0)
	{
	  //left epidermis
	  i=disttoepi+0.5*totsizeepi;
	  sprintf(s,"%s/Spatialdynamics_epiL_time%i",dirname,t);
	}
      else if(k==1)
	{
	  //right epidermis
	  ii=disttoepi+0.5*totsizeepi;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_epiR_time%i",dirname,t);
	}
      else if(k==2)
	{
	  //left cortex
	  i=disttoepi+totsizeepi+0.5*totsizecort;
	  sprintf(s,"%s/Spatialdynamics_cortexL_time%i",dirname,t);
	}
      else if(k==3)
	{
	  //right cort
	  ii=disttoepi+totsizeepi+0.5*totsizecort;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_cortexR_time%i",dirname,t);
	}
      else if(k==4)
	{
	  //left endodermis
	  i=disttoepi+totsizeepi+totsizecort+0.5*totsizeendo;
	  sprintf(s,"%s/Spatialdynamics_endoL_time%i",dirname,t);
	}
      else if(k==5)
	{
	  //right endodermis
	  ii=disttoepi+totsizeepi+totsizecort+0.5*totsizeendo;
	  i=n-1-ii;
	  sprintf(s,"%s/Spatialdynamics_endoR_time%i",dirname,t);
	}
      else if(k==6)
	{
	  //left vasculature
	  i=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+0.3*totsizevasc;
	  sprintf(s,"%s/Spatialdynamics_vascL_time%i",dirname,t);
	}
      else if(k==7)
	{
	  //right vasculature
	  ii=disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+0.3*totsizevasc;
	  i=n-1-i;
	  sprintf(s,"%s/Spatialdynamics_vascR_time%i",dirname,t);
	}
      
      
      g=fopen(s,"wb");
      for(j=0;j<m;j++)
	{
	  if(CellGrid[i][j]!=NULL)
	    {
	      c=CellGrid[i][j];
	      fprintf(g,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n",j,c->MeanCells,c->ARF,c->AA,c->BB,c->CC,c->DD,c->Plts,c->Pltsexpr,c->Diff,c->CellHeight);
	    }
	  else if(Tc[i][j]==3)
	    fprintf(g,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n",j,-10.0,-10.0,-10.0,-10.0,-10,-10.0,-10.0,-10.0,-10.0,-10);
	}
      fclose(g);
      
    }
}

void Tissue::SaveGradGridRes(int time)
{
  FILE *ff1;
  FILE *ff2;
  char s1[900];
  char s2[900];
  int i,ii,j,k;
  int List[8]={1,5,6,7,8,100,100,100};
  
  for(k=0;k<8;k++)
    {	
      if(k==0)
	{
	  sprintf(s1,"%s/GridResolutionL_RC1_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_RC1_time%i",dirname,time);
	}
      if(k==1)
	{
	  sprintf(s1,"%s/GridResolutionL_Epi_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Epi_time%i",dirname,time);
	  
	}
      if(k==2)
	{
	  sprintf(s1,"%s/GridResolutionL_Cort_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Cort_time%i",dirname,time);
	  
	}
      if(k==3)
	{
	  sprintf(s1,"%s/GridResolutionL_Endo_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Endo_time%i",dirname,time);
	  
	}
      if(k==4)
	{
	  sprintf(s1,"%s/GridResolutionL_Peri_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Peri_time%i",dirname,time);
	  
	}
      if(k==5)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc1_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc1_time%i",dirname,time);
	  
	}
      if(k==6)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc2_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc2_time%i",dirname,time);
	  
	}
      if(k==7)
	{
	  sprintf(s1,"%s/GridResolutionL_Vasc3_time%i",dirname,time);
	  sprintf(s2,"%s/GridResolutionR_Vasc3_time%i",dirname,time);
	  
	}
      
      
      
      
      ff1=fopen(s1,"w");
      ff2=fopen(s2,"w");	
      
      
      /*
      for(j=0;j< yLRCstraight;j++)//special treatment for curved part
	{
	  
	  i=0;
	  
	  while(Tct[i][j]!=List[k]  && i<xm)//find position of right cell type
	    i++;
	  
	  if(Tct[i][j]==List[k])
	    {
	      i++;
	      ii=n-(1+i);
	      
	      if(CellGrid[i][j]!=NULL)
		{
		  fprintf(ff1,"%i\t%i\t%f\t%f\t%f\n",i,j,Caux1[i][j],Cpin1M[i][j], Aux1M[i][j]);
		  fprintf(ff2,"%i\t%i\t%f\t%f\t%f\n",ii,j,Caux1[ii][j],Cpin1M[ii][j], Aux1M[ii][j]);
		}
	    }
	    }*/

      for(j=yLRCstraight;j<yEND;j++)//for(j= yQC;j<yEND;j++)//yLRCstraight
	{
	  if(k==0)
	    i=1;
	  else if(k==1)
	    i=disttoepi+1;
	  else if(k==2)
	    i=totsizeepi+disttoepi+1;
	  else if(k==3)
	    i=totsizecort+totsizeepi+disttoepi+1;
	  else if(k==4)
	    i=totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==5)
	    i=totsizeperi+totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==6)
	    i=sizevasc+totsizeperi+totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  else if(k==7)
	    i=xm-1;//2*sizevasc+totsizeperi+totsizeendo+totsizecort+totsizeepi+disttoepi+1;
	  
	  ii=n-(1+i);
	  if(CellGrid[i][j]!=NULL)
	    {
	      fprintf(ff1,"%i\t%i\t%f\t%f\t%f\n",i,j,Caux1[i][j],Cpin1M[i][j], Aux1M[i][j]);
	      fprintf(ff2,"%i\t%i\t%f\t%f\t%f\n",ii,j,Caux1[ii][j],Cpin1M[ii][j], Aux1M[ii][j]);
	    }
	}
      
      fclose(ff1);
      fclose(ff2);   
	  		
    }		             
 }   

    
    
