#include "Tissue.hh"

#include "BCell.hh"

void Tissue::Tdsolvetridiag(int ni)
{

  /**
   * ni - number of equations
   * a - sub-diagonal (means it is the diagonal below the main diagonal) -- indexed from 1..n-1
   * b - the main diagonal
   * c - sup-diagonal (means it is the diagonal above the main diagonal) -- indexed from 0..n
   * rhs - right part
   * x - the answer
   */
  int i;
  double mm;

  for (i = 1; i < ni; i++)
    {
      mm = a[i]/b[i-1];
      b[i] = b[i] - mm*c[i-1];
      rhs[i] = rhs[i] - mm*rhs[i-1];
    }

  res[ni-1] = rhs[ni-1]/b[ni-1];

  for (i = ni - 2; i >= 0; i--)
    res[i]=(rhs[i]-c[i]*res[i+1])/b[i];

  for(i=0;i<ni;i++)
    rhs[i]=res[i];

}


void Tissue::UpdateAuxinTissue()
{
  int i,j;
  static int Direction=0;
  int q;
  double func;
 
  double invspstep=1.0/(double)(xstep);

  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction

      Direction=1;
      //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1
		  b[j]=1-(a[j]+c[j]);//j
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }

		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]);
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  Tdsolvetridiag(m);
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
    }
  if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction

      Direction=0;

      //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }


		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  Tdsolvetridiag(n);
	  for(i=0;i<n;i++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  iter it;
  BCell *c;
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  (*it)->MeanCells=0;
	  it++;
	}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)
	  {
	    c=CellGrid[i][j];
	    c->MeanCells+=Caux1[i][j];
	  }
      }
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  if((*it)->CellSize>0)
	    (*it)->MeanCells=(*it)->MeanCells*(*it)->InvCellSize;
	  it++;
	}
    }

  //final update for next timestep
  //doe dit pas na het berekenen van het
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }

#ifdef NEW3D2
 
  int ii;
  double invreldistance;
  double influx1,influx2;
  double efflux1,efflux2;
  double percbase=0.5*0.05*16./46.0;
  double exchange,exchange1,exchange2;

  j=yLRCstraight;
  i=disttoepi;
  while(Tc[i][j]!=3)
    {
      for(i=disttoepi;i<xm;i++)
	{
	  ii=n-1-i;
	  invreldistance=(xm-disttoepi)/(xm-i);
	  
	  if(Tc[i][j]==0)//wall
	    {
	      exchange=(Caux1[i][j]-Caux1[ii][j]);
	      exchange*=invreldistance*percbase;
	      Caux1[i][j]=Caux1[i][j]-exchange;
	      Caux1[ii][j]=Caux1[ii][j]+exchange;
	    }
	  else//membrane or cell
	    {
	      if(Tct[i][j]>=8 && Tct[i][j]<=11)
		{
	      c=CellGrid[i][j];
	      influx1=aux1fac*c->Aux1;//+aux1fac*c->Lax3+inf;
	      efflux1=1.;//pPUMP*c->Pin+eff;
	      c=CellGrid[ii][j];
	      influx2=aux1fac*c->Aux1;//+aux1fac*c->Lax3+inf;
	      efflux2=1;//pPUMP*c->Pin+eff;
	      exchange1=((influx1/(influx1+influx2))*(Caux1[i][j]*efflux1+Caux1[ii][j]*efflux2)-Caux1[i][j]*efflux1);
	      exchange2=((influx2/(influx1+influx2))*(Caux1[i][j]*efflux1+Caux1[ii][j]*efflux2)-Caux1[ii][j]*efflux2); 
	      
	      //exchange1=-(influx2-influx1)*0.5*(Caux1[i][j]+Caux1[ii][j]);
	      //exchange2=(influx2-influx1)*0.5*(Caux1[i][j]+Caux1[ii][j]);
	      exchange1*=invreldistance*percbase*0.3;//*0.1;
	      exchange2*=invreldistance*percbase*0.3;//*0.1;
	      Caux1[i][j]=Caux1[i][j]+exchange1;
	      Caux1[ii][j]=Caux1[ii][j]+exchange2;
		}
	    }
	}
      i=disttoepi;
      j++;
    }

#endif
#ifdef NEW3D
  j=yLRCstraight;
  i=disttoepi;
  int ii;
  double invreldistance;
  double effinfratio1;
  double effinfratio2;
  double percbase=0.5*0.05*16./46.0;
  double exchange;
  while(Tc[i][j]!=3)
    {
      ii=n-1-i;
      invreldistance=(xm-disttoepi)/(xm-i);

      if(Tc[i][j]==0)//wall
	{
	  exchange=(Caux1[i][j]-Caux1[ii][j]);
	  exchange*=invreldistance*percbase;
	  Caux1[i][j]=Caux1[i][j]-exchange;
	  Caux1[ii][j]=Caux1[ii][j]+exchange;
	}
      else//membrane or cell
	{
	  c=CellGrid[i][j];
	  effinfratio1=(pPUMP*c->Pin+eff)/((aux1fac*c->Aux1+aux1fac*c->Lax3+inf)*(aux1fac*c->Aux1+aux1fac*c->Lax3+inf));
	  c=CellGrid[ii][j];
	  effinfratio2=(pPUMP*c->Pin+eff)/((aux1fac*c->Aux1+aux1fac*c->Lax3+inf)*(aux1fac*c->Aux1+aux1fac*c->Lax3+inf));
	  exchange=(effinfratio1*Caux1[i][j]-effinfratio2*Caux1[ii][j]);
	  exchange*=invreldistance*percbase;
	  Caux1[i][j]=Caux1[i][j]-exchange;
	  Caux1[ii][j]=Caux1[ii][j]+exchange;
	}
      j++;
    }

#endif


#ifdef PERI3D
  int leftepiout=disttoepi;
  int leftcortout=disttoepi+totsizeepi;
  int leftendoout=disttoepi+totsizeepi+totsizecort;
  int rightepiout=n-1-leftepiout;
  int rightcortout=n-1-leftcortout;
  int rightendoout=n-1-leftendoout;
  int leftperiout=disttoepi+totsizeepi+totsizecort+totsizeendo;
  int leftperiin=leftperiout+totsizeperi;
  int rightperiin=n-1-leftperiin;
  int rightperiout=n-1-leftperiout;
  double perc;

  //debugged kirsten 7 feb
  double exchange;
  double percbase=0.5*0.05*16./46.0;

  j=yLRCstraight;
  i=xm-2;
  while(Tc[i][j]!=3)
    {
      perc=((double)(xm-leftepiout)/(double)(xm-leftepiout))*percbase;
      exchange=perc*(Caux1[leftepiout][j]-Caux1[rightepiout][j]);
      Caux1[leftepiout][j]=Caux1[leftepiout][j]-exchange;
      Caux1[rightepiout][j]=Caux1[rightepiout][j]+exchange;

      perc=((double)(xm-leftepiout)/(double)(xm-leftcortout))*percbase;
      exchange=perc*(Caux1[leftcortout][j]-Caux1[rightcortout][j]);
      Caux1[leftcortout][j]=Caux1[leftcortout][j]-exchange;
      Caux1[rightcortout][j]=Caux1[rightcortout][j]+exchange;

      perc=((double)(xm-leftepiout)/(double)(xm-leftendoout))*percbase;
      exchange=perc*(Caux1[leftendoout][j]-Caux1[rightendoout][j]);
      Caux1[leftendoout][j]=Caux1[leftendoout][j]-exchange;
      Caux1[rightendoout][j]=Caux1[rightendoout][j]+exchange;

      perc=((double)(xm-leftepiout)/(double)(xm-leftperiout))*percbase;
      exchange=perc*(Caux1[leftperiout][j]-Caux1[rightperiout][j]);
      Caux1[leftperiout][j]=Caux1[leftperiout][j]-exchange;
      Caux1[rightperiout][j]=Caux1[rightperiout][j]+exchange;

      perc=((double)(xm-leftepiout)/(double)(xm-leftperiin))*percbase;
      exchange=perc*(Caux1[leftperiin][j]-Caux1[rightperiin][j]);
      Caux1[leftperiin][j]=Caux1[leftperiin][j]-exchange;
      Caux1[rightperiin][j]=Caux1[rightperiin][j]+exchange;

      j++;
    }
#endif



  //#define VASC3D
#ifdef VASC3D
  //do artificial semi-3D thing:
  //make part of auxin from inside wall endodermis
  //not go to pericycle but go to middle of vasculature
  i=disttoepi+totsizeepi+totsizecort+totsizeendo;
  int ii=n-1-i;
  j=yLRCstraight;


  double temp1,temp2;
  while(Tc[i][j]!=3)//for(j=yLRCstraight;j<m;j++)
    {
      temp1=tstep*0.25*Caux1[i][j];//0.75
      temp2=tstep*0.25*Caux1[ii][j];//0.5
      Caux1[i][j]-=temp1;
      Caux1[ii][j]-=temp2;
      Caux1[xm][j]+=temp1+temp2;
      j++;
    }
#endif

}

/**********************************************************************/

void Tissue::UpdateGeneExpressionTissue(int t)
{
  iter it;
  BCell *c;
  BCell *cdown;
  BCell *cdown2;
  BCell *cup;
  BCell *cup2;
  double kmahp6=40;//35;//40;//lower than km of CK?
  double kmckl=40;
  double kms1=35;
  double kms2=35;
  double kmdif=2.5;//15;
  double dtAHP6;
  double dtCKlevel;
  double threshold=0.1;
  static double tspeedupstep=tstep*speedup2;


  int lengthtotal;


  double pahp6=0.033;
  double dahp6=0.00033;
  double pck=3*0.033;
  double dck=0.00033;
  double pcksignal=0.033;
  double dcksignal=0.00033;


  double sameRate=0.02;//0.002;
  double pUp=sameRate;
  double dUp=sameRate;
  double pDown=sameRate;
  double dDown=sameRate;
  double pIn=sameRate;
  double dIn=sameRate;

  double diffusionCK=6;
  double diffusion;


  //possible issues with polarisation of pericycle:
  //1. due to cell size increase general tendency of auxin increase when moving shootward
  //try now to avoid it by starting with polarisation not at start of EZ but probably still not okay
  //2. if pericycle does not pump down any more possibly lot of auxin loss for root tip
  //try now to avoid it by have define 3DVASCULATURE on and having 0.25 of inner endodermis wall
  //auxin pumped to the middle of the vasculature
  //3. numerical issues, auxin seems to go to plus/minus infinity upon PIN repolarisation


  BCell *cup3;
  BCell *cdown3;
  BCell *clateral;//
  double meannbARF;

  //actual gene expression
  double diffact;
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  //if((*it)->CellSize>0)
	    {
	      
	      (*it)->UpdateCell(t);


	      (*it)->IAAreg=1;
	      //lateral inhibition
	      
	      /*
	      if((*it)->CellMinJ>yLRC1)
		{
		  cup=CellGrid[(*it)->CellMinI][(*it)->CellMaxJ+2];
		  if(cup!=NULL)
		    {
		      cup2=CellGrid[cup->CellMinI][cup->CellMaxJ+2];
		      if(cup2!=NULL)
			cup3=CellGrid[cup2->CellMinI][cup2->CellMaxJ+2];
		      else
			cup3=NULL;
		    }
		  else
		    cup2=NULL;
		  cdown=CellGrid[(*it)->CellMinI][(*it)->CellMinJ-2];
		  cdown2=CellGrid[cdown->CellMinI][cdown->CellMinJ-2];
		  cdown3=CellGrid[cdown2->CellMinI][cdown2->CellMinJ-2];
		
		  if(Tct[(*it)->CellMinI][(*it)->CellMinJ]==8)
		    {
		      clateral=CellGrid[n-1-(*it)->CellMinI][(*it)->CellMinJ];                          
		    }
		  else
		    clateral=NULL;
		  
		  meannbARF=(*it)->memARF;
		  meannbARF+=0.5*cdown->memARF;
		  meannbARF+=0.25*cdown2->memARF;
		  meannbARF+=0.125*cdown3->memARF;
		  if(cup!=NULL)
		    {
		      meannbARF+=0.5*cup->memARF;
		      if(cup2!=NULL)
			{
			  meannbARF+=0.25*cup2->memARF;
			  if(cup3!=NULL)
			    meannbARF+=0.125*cup3->memARF;
			}
		    }
		  if(clateral==NULL)
		    meannbARF/=2.75;
		  else
		    {
		      meannbARF+=0.5*clateral->memARF;
		      meannbARF/=3.25;
		    }
		  (*it)->IAAreg=1+4.5*(meannbARF*meannbARF/(meannbARF*meannbARF+70*70))*(70*70/(70*70+(*it)->memARF*(*it)->memARF));
		   
		}
	      */


#ifdef SIMPLEPOLARISATION
	      c=(*it);
	      if(Tct[c->CellMinI][c->CellMinJ]==8 )//perhaps replace by Diff th
		{
		  if(c->Diff>80)
		    {
		       cdown=CellGrid[c->CellMinI][c->CellMinJ-2];
		       cup=CellGrid[c->CellMinI][c->CellMaxJ+2];
		       if(cup!=NULL)
			 cup2=CellGrid[cup->CellMinI][cup->CellMaxJ+2];
		       else
			 cup2=NULL;
		       
		       if(cup!=NULL && cup2!=NULL)
			 {
			   if(cup->epiO>125)
			     {
			       //SO THIS POLARIZES THE CELL BELOW THE PRIMED CELL
			       double difdif;
			       if(c->epiO<cup->epiO && cup->epiO>cup2->epiO)
				 difdif=0.5;//0.25;//1;
			       else
				 difdif=0;
			       
			       c->dpinup=tspeedupstep*(pUp*0.7*(difdif)-dUp*c->pinup);
			       c->dpindown=tspeedupstep*(pDown*0.7*(1-difdif)-dDown*c->pindown);
			       c->dpinin=0;
			     }
			   else
			     {
			       c->dpinup=0;
			       c->dpindown=0;
			       c->dpinin=0;
		     }                              
			 }
		    }
		}
#endif 

	    }//cell size larger than zero
	  it++;
	}//list not at end
    }//list not null


 if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  //if((*it)->CellSize>0)
	    {
	      //(*it)->UpdateCell(t);//WHY HERE, WAS DONE ALREADY ABOVE!
#ifdef SIMPLEPOLARISATION
	      c=(*it);
	      if(Tct[c->CellMinI][c->CellMinJ]==8 )//perhaps replace by Diff th
		{
		  if(c->Diff>25)
		    {		      
		      c->pinup+=c->dpinup;
		      c->pindown+=c->dpindown;
		      c->pinin+=c->dpinin;
		    }
		} 
#endif
	    }//cell size larger than zero
	  it++;
	}//list not at end
    }//list not null



  //flux of proteins through plasmodesmata
#ifdef PLASMODESMATAOPEN
  int i,j,ii,jj;
  double pdflux=4*0.01/(double)xstep;
  //initialise PLTs on grid
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//cell there
	  {
	    c=CellGrid[i][j];
	    Pltsgrid[i][j]=c->TOLS2;//c->Plts;
	    Pltsgrid2[i][j]=c->TOLS2;//c->Plts;
	  }
	else
	  {
	    Pltsgrid[i][j]=0;
	    Pltsgrid2[i][j]=0;
	  }
      }

  //PLTs fluxes between cells through plasmodesmata
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//mebrane, so point of possible exchange
	  {
	    if(Tc[i-1][j]==0)//wall to left, so left membrane
	      {
		ii=i-1;
		while(ii>0 && (i-ii)<4)
		  {
		    ii--;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i+1][j]==0)//wall to right, so right membrane
	      {
		ii=i+1;
		while(ii<n && (ii-i)<4)
		  {
		    ii++;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j-1]==0)//wall above, so apical membrane ???????
	      {
		jj=j-1;
		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=5*pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j+1]==0)//wall below, so basal membrane ?????????
	      {
		jj=j+1;
		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=5*pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	  }
      }
#ifdef PLTLEAKAGETOMAINROOT
  double Pltslevelshoot=0;
  double pdfluxshoot=pdflux;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	if(CellGrid[i][j]!=NULL)//topmost part of topmost cell found
	  {
	    Pltsgrid2[i][j]+=pdfluxshoot*(Pltslevelshoot-Pltsgrid[i][j]);
	    j=-100;//so break from second for loop
	  }
      }
#endif


  //transform back from grid to cellular
  //assumption that within cell PLTs differences
  //even out quickly through diffusion
  if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  (*it)->TOLS2=0;//(*it)->Plts=0;
	  it++;
	}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//cell there
	  {
	    c=CellGrid[i][j];
	    c->TOLS2+=Pltsgrid2[i][j];//c->Plts+Pltsgrid2[i][j];//BUG, CHECK MONICA CODE?
	  }
      }
#ifdef EPIGFEEDBACK
   if(CellList!=NULL)
    {
      it=CellList->begin();
      while(it!=CellList->end())
	{

	  if((*it)->CellSize>0)
	    {
	      (*it)->TOLS2=(*it)->TOLS2*(*it)->InvCellSize;//(*it)->Plts=(*it)->Plts*(*it)->InvCellSize;
#ifdef TOLS2LATERALINHIBITION
	      (*it)->IAAreg=1+2.5*((*it)->TOLS2*(*it)->TOLS2/((*it)->TOLS2*(*it)->TOLS2+50*50))*(60*60/(60*60+(*it)->memARF*(*it)->memARF));
#endif

	      //(*it)->IAAreg=1+2.5*((*it)->TOLS2*(*it)->TOLS2/((*it)->TOLS2*(*it)->TOLS2+50*50))*(60*60*60*60/(60*60*60*60+(*it)->memARF*(*it)->memARF*(*it)->memARF*(*it)->memARF));
	      //higher power to protect yourselves?
	    }
	  it++;
	}
    }
#endif
#endif
}


void Tissue::UpdateFluxesTissue(int t)
{
  BCell *c;
  BCell *cacross;
  iter it;
  int i,j,k,ii;
  double invspstep=1.0/(double)(xstep);

  BCell *cos;
  BCell *cout;
  BCell *cin;


  //growth -> location cells -> zones
  //zones -> transporter prepatterns

   for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	ii=n-1-i;
	PinPrePat[i][j]=0;
	PinPrePat[ii][j]=0;
	Aux1PrePat[i][j]=0;
	Aux1PrePat[ii][j]=0;
	Lax3PrePat[i][j]=0;
	Lax3PrePat[ii][j]=0;
      }


#ifndef RESTART
  if(t==0)
    {
      if(CellList!=NULL)
	{
	  it=CellList->begin();
	  while(it!=CellList->end())
	    {

	      if((*it)->CellMaxJ<yR)
		(*it)->CellZonationState=0;
	      else if((*it)->CellMaxJ<yLRC1-(15*HMZ+15))
		(*it)->CellZonationState=30;
	      else if((*it)->CellMaxJ<yLRC1)
		(*it)->CellZonationState=50;
	      else if((*it)->CellMaxJ<yLRC1+(7*HEZ+7))
		(*it)->CellZonationState=60;
	      else
		(*it)->CellZonationState=100;

	      it++;
	    }
	}
    }
#endif



  int kk,ll;
  int mindist;
  double disttresh;
  double distance;
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    ii=n-1-i;
	    c=CellGrid[i][j];
	    cacross=CellGrid[ii][j];
	    if(Tct[i][j]<=4) //lateral root cap
	      {		
		if(Tct[i][j]==1 && j>yLRCstraight) //lateral root cap
		  {
		    if(Tmt[i][j]==1)//up
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(Tmt[i][j]==3)//inward membrane
		      {
			disttresh=(int)(0.2*(c->CellMaxJ-c->CellMinJ));
			distance =10000;
			for(kk=max(0,i-100/xstep);kk<=min(n-1,i+100/xstep);kk++) //min max to not go outside the field
			  for(ll=c->CellMinJ;ll<=c->CellMaxJ;ll++)//cells always inside the field
			    {
			      if(Tmt[kk][ll]==1  && CellGrid[kk][ll]==CellGrid[i][j])//upward membrane of same cell
				{
				  distance=abs(kk-i)+abs(ll-j);
				  if(distance<=disttresh)
				    {
				      kk=10000;
				      ll=10000;//to stop searching if distance below threshold is found
				    }
				}
			    }
			if(distance<=disttresh)
			  {
			    PinPrePat[i][j]=1;
			    PinPrePat[ii][j]=1;
			  }
			else
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      }
		  }
		else //other lateral root cap cells
		  {
		    if(Tmt[i][j]==1)//up
		      {
			if((i<xm && j>=(int)(arad1*i+brad1)) || (i>xm && j>=(int) (yR-arad1*(i-xm))))
			  {
			    PinPrePat[i][j]=3;//1;//WHY NOT 1???
			    PinPrePat[ii][j]=3;//1;//1.5;
			  }
			else
			  {
			    PinPrePat[i][j]=1.5;//1;//WHY NOT 1???
			    PinPrePat[ii][j]=1.5;//1;//1.5;
			  }
		      }
		    else if(Tmt[i][j]==3 && ((i<xm && j>=(int)(arad1*i+brad1)) || (i>xm && j>=(int) (yR-arad1*(i-xm)))))
		      {
			PinPrePat[i][j]=0.3;//0.25;//0.05;
			PinPrePat[ii][j]=0.3;//0.25;//0.05;
			
		      }
		    else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		      {
			PinPrePat[i][j]=0.05;//0.25;//0.05;
			PinPrePat[ii][j]=0.05;//0.25;//0.05;
		      } 
		  } 
	      }
	    else if(Tct[i][j]==5)//epidermis
	      {
		if(Tmt[i][j]==1)//up
		  {
		    if(j<yQC+10*HMZ+10)//low in MZ
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else if(c->CellZonationState<60)//untill MZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else//DZ
		      {
			PinPrePat[i][j]=0.35;     
			PinPrePat[ii][j]=0.35;   
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		  {
		    if(Tmt[i][j]==3)//in
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		    if(Tmt[i][j]==4 && c->CellMinJ<yQC+20*HMZ+20)//out to lrc
		      {
			PinPrePat[i][j]=0.3;
			PinPrePat[ii][j]=0.3;
		      }
		    else if(Tmt[i][j]==4 && c->CellZonationState<=50)//Thea?
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		if(Tmt[i][j]==1 && c->CellZonationState>50)//up: beyond MZ
		  {
		    if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else//DZ
		      {
			PinPrePat[i][j]=0.35;   
			PinPrePat[ii][j]=0.35;     
		      }
		  }
		else if(Tmt[i][j]==2 && c->CellZonationState<=50)//down: in MZ//FLIP AT TZ!!!
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
		else if(Tmt[i][j]==4 && c->CellZonationState>60)
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==7)//endodermis
	      {
		if(Tmt[i][j]==2)//down
		  {
		    if(c->CellZonationState<60)//till MZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else
		      {
			PinPrePat[i][j]=0.35; 
			PinPrePat[ii][j]=0.35; 
		      }
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		    else
		      {
			PinPrePat[i][j]=0.15;
			PinPrePat[ii][j]=0.15;
		      }
		  }
		else if((Tmt[i][j]==4 || Tmt[i][j]==1) && c->CellZonationState==100)//out
		  {
		    PinPrePat[i][j]=0.15; 
		    PinPrePat[ii][j]=0.15; 
		  }
	      }
	    else if(Tct[i][j]==8)//pericycle
	      {

		if(Tmt[i][j]==2)//down
		  {
		    if(c->CellZonationState<60)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.9;//0.8;//0.9;
			PinPrePat[ii][j]=0.9;//0.8;//0.9;
		      }
		    else
		      {
			PinPrePat[i][j]=0.7;//0.6;//0.7;
			PinPrePat[ii][j]=0.7;//0.6;//0.7;
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  {
		    if(c->CellMinJ<yQC+5*HMZ+5)
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else if(Tmt[i][j]==3 )
		      {
			if(c->CellZonationState<100)
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      }
		  }
	      }
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature
	      {
		if(Tmt[i][j]==2)//down
		  {
		    if(c->CellZonationState<60)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.8;//0.6;
			PinPrePat[ii][j]=0.8;//0.6;
		      }
		    else
		      {
			PinPrePat[i][j]=0.6;//0.35;   //0.6//DZequaltoEZ//0.5
			PinPrePat[ii][j]=0.6;//0.35;    //0.6//DZequaltoEZ//0.5
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  {
		    if(c->CellMinJ<yQC+5*HMZ+5)
		      {
			PinPrePat[i][j]=0.4;
			PinPrePat[ii][j]=0.4;
		      }
		    else //if(Tmt[i][j]==4)//17 april 2019
		      {
			if(Tmt[i][j]==4)//7 mei
			  {
			    PinPrePat[i][j]=0.1;//0.15;//17 april 2019
			    PinPrePat[ii][j]=0.1;//0.15;//17 april 2019
			  }
		   
		    }
		}
	      }
	    else if(Tct[i][j]==12)//QC
	      {
		if(j==c->CellMinJ)//works better than Tmt[i][j]==4
		  {
		    PinPrePat[i][j]=0.5;//2;//0.5;
		    PinPrePat[ii][j]=0.5;//2;//0.5;
		  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer
	      {
		if(Tmt[i][j]<4)
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]>13)//columella except bottom layer
	      {
		if(Tct[i][j]<17)//not the topmost one
		  {

		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
	      }
	  }
      }

#ifdef PIN2MUTANT
   for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]==1 || Tct[i][j]>=5 && Tct[i][j]<=7)//
	      PinPrePat[i][j]*=0.4;
	  }
      }
#endif

#ifdef SIMPLEPOLARISATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]==8)//pericycle
	      {
		c=CellGrid[i][j];
		//if(c->CKsignal>5 && c->auxindif>0)//WHAT HAPPENS NOW, WORSE OR BETTER?
		  {
		    if(Tmt[i][j]==1)
		      PinPrePat[i][j]=c->pinup;
		    else if(Tmt[i][j]==2)
		      PinPrePat[i][j]=c->pindown;
		    else if(Tmt[i][j]==3)
		      PinPrePat[i][j]=c->pinin;
		  }
	      }
	  }
      }
#endif

#ifdef AHP6SIGNALLING
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]==8)//pericycle
	      {
		c=CellGrid[i][j];
		if(c->CKsignal>5 && c->auxindif>0)//WHAT HAPPENS NOW, WORSE OR BETTER?
		  {
		    if(Tmt[i][j]==1)
		      PinPrePat[i][j]=c->pinup;
		    else if(Tmt[i][j]==2)
		      PinPrePat[i][j]=c->pindown;
		    else if(Tmt[i][j]==3)
		      PinPrePat[i][j]=c->pinin;
		  }
	      }
	  }
      }
#endif




  //AUX1
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    ii=n-1-i;
	    c=CellGrid[i][j];

	    if(Tct[i][j]>=12 && Tct[i][j]!=13)//QC+nb columella layer
	      {
		Aux1PrePat[i][j]=3;
	      }
	    else if(Tct[i][j]==13 || (Tct[i][j]>=1 && Tct[i][j]<=4))//lowest columella layer + lrc1-4
	      {
		Aux1PrePat[i][j]=1;
	      }
	    else if(Tct[i][j]==5)//epi
	      {
		if(c->CellMaxJ<yMZEZ-4*HMZ-4)
		  Aux1PrePat[i][j]=0.1;
		else if(c->CellMaxJ<yMZEZ)
		    Aux1PrePat[i][j]=0.6;
		else
		    Aux1PrePat[i][j]=1;		  
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		if(c->CellMaxJ<yMZEZ-4*HMZ-4)
		  Aux1PrePat[i][j]=0.1;
		else
		  Aux1PrePat[i][j]=0.5;
	      }
	    else if(Tct[i][j]>=8 && Tct[i][j]<=11)//peri and vasc
	      {
		if(c->CellMaxJ<yQC+10*HMZ+10)
		  Aux1PrePat[i][j]=2;
		else
		  Aux1PrePat[i][j]=1;
	      }
	  }
      }


  //LAX3
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	double h;

	if(Tc[i][j]==2)//membrane there?
	  {
	    ii=n-1-i;
	    c=CellGrid[i][j];

	    if((Tct[i][j]>7 && Tct[i][j]<12))//peri-vasc
	      {
		if(j<=yLRCstraight+10*HMZ+10)
		  Lax3PrePat[i][j]=0.1;
		else if(j>yLRCstraight+10*HMZ+10 && j<yLRCstraight+10*HMZ+10+20*HMZ+20)
		  {
		    h=j-(yLRCstraight+10*HMZ+10);
		    h=h/(HMZ+1);
		    Lax3PrePat[i][j]=0.045*h+0.1;
		  }
		else
		  Lax3PrePat[i][j]=1;
	      }
	    else if (Tct[i][j]==6 || Tct[i][j]==5)//epi/cort  
	      {
		if(c->CellZonationState>60)//only above MZ
		  {

		    Lax3PrePat[i][j]=1;
		    if(Tct[i][j]==6)
		      Lax3PrePat[i][j]=0.75;
		  }
	      }
	    else
	      Lax3PrePat[i][j]=0;
	  }
      }




  //gene expression * prepatterns-> transporter membrane levels
  //update membrane protein concentration based on new prepatterns and gene expression levels
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//grid point inside cell
	  {
	    c=CellGrid[i][j];
	    Cpin1M[i][j]=c->Pin*PinPrePat[i][j];
	    Aux1M[i][j]=c->Aux1*Aux1PrePat[i][j];
	    Lax3M[i][j]=c->Lax3*Lax3PrePat[i][j];
	  }
	else
	  {
	    Cpin1M[i][j]=0;
	    Aux1M[i][j]=0;
	    Lax3M[i][j]=0;
	  }
      }
  //membrane levels -> fluxes
  //direction 0 i+1;
  //direction 1 i-1;
  //direction 2 j+1;
  //direction 3 j-1;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	daux[i][j]=0;//default no decay
	paux[i][j]=0;//default no production
	for(k=0;k<4;k++)
	  {
	    Inf[i][j][k]=0;
	    Eff[i][j][k]=0;
	    DiffAuxC[i][j][k]=0;
	  }
      }
  //daux depends on Gh33 gene expression
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	c=CellGrid[i][j];
	if(c!=NULL)//homeostatis
	  {
	    daux[i][j]=0.65*dAux;//+40*(c->memARF*c->memARF/(c->memARF*c->memARF+75*75))*0.65*dAux;//+30*(c->memARF*c->memARF/(c->memARF*c->memARF+100*100))*0.65*dAux;//+25*(c->memARF*c->memARF/(c->memARF*c->memARF+100*100))*0.65*dAux;//+10*(c->memARF*c->memARF/(c->memARF*c->memARF+100*100))*0.65*dAux;;//+7.5*(c->memARF*c->memARF/(c->memARF*c->memARF+75*75))*0.65*dAux;;//+5*(c->memARF*c->memARF/(c->memARF*c->memARF+50*50))*0.65*dAux;//+3*(c->memARF/300.)*0.65*dAux;
	//gh33 fac=0.7 so remainder is 0.3
	//Gh33 is 50, so gh33fac*0.01*50=0.7*0.5=0.35
	//so decayrate =0.65*dAux
	  }
      }


  //paux depends on Yuc3 expression and cell type
  double volfactor;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
	{
	  ii=n-1-i;
	  c=CellGrid[i][j];
	  if(c!=NULL)
	    {
	      if(Tct[i][j]==12 || Tct[i][j]==17)//QC and columella layer 5
		volfactor=50*2;
	      else if(j<yQC+2*HMZ+2 && (Tct[i][j]>=8 && Tct[i][j]<=11))
		volfactor=50*2;
	      else if(Tct[i][j] > 12 && Tct[i][j]< 17 )//Columella layers 1-4
		volfactor = 25*2;
	      else if(Tct[i][j]>=1 && Tct[i][j]<=4)//Lateral Root Cap
		volfactor=15*2;
	      else
		volfactor=1;
	      /*
		{
		  if(c->CellZonationState<100)
		    volfactor=1;
		  else
		    volfactor=0;
		}
	      */
	      paux[i][j]=(volfactor*0.65*pAux+5*5*0.01*c->Yuc3*pAux)*HMZ/c->CellHeight;
	    }
	}
#ifdef AUXINAPPLICATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(i==0 || i==n-1 || j==0)
	  paux[i][j]+=1.00/xstep;
      }
#endif



  //#define VOLUME3D
  double r, rmin;
  double v=1;
  double vf=0.5;//1;
  //Set Inf from Lax3,Lax2,Aux1
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)
	  {
	    //nearest cell:
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
			  
			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  Inf[i][j][1]=v*(-inf-aux1fac*Aux1M[i-1][j]-aux1fac*Lax3M[i-1][j])*invspstep;
		}
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  Inf[i][j][0]=v*(-inf-aux1fac*Aux1M[i+1][j]-aux1fac*Lax3M[i+1][j])*invspstep;
		}
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Inf[i][j][3]=(-inf-aux1fac*Aux1M[i][j-1]-aux1fac*Lax3M[i][j-1])*invspstep;
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Inf[i][j][2]=(-inf-aux1fac*Aux1M[i][j+1]-aux1fac*Lax3M[i][j+1])*invspstep;
	  }
	else if(Tc[i][j]==2)
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);

			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  Inf[i][j][0]=v*(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);

			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  Inf[i][j][1]=v*(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Inf[i][j][2]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Inf[i][j][3]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	  }
      }
  //Set Eff from Tc
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall, neighboring membrane
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);

			  /*
			   if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  Eff[i][j][0]=v*(eff*invspstep+pPUMP*(Cpin1M[i+1][j])*invspstep);
		}
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
			  
			  /*
			   if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  Eff[i][j][1]=v*(eff*invspstep+pPUMP*(Cpin1M[i-1][j])*invspstep);
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Eff[i][j][2]=eff*invspstep+pPUMP*(Cpin1M[i][j+1])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Eff[i][j][3]=eff*invspstep+pPUMP*(Cpin1M[i][j-1])*invspstep;
	  }
	else if(Tc[i][j]==2)//membrane, neighboring wall
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
			  
			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  Eff[i][j][0]=v*(-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep);
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
			  
			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  Eff[i][j][1]=v*(-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep);
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Eff[i][j][2]=-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Eff[i][j][3]=-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep;
	  }
      }
  //Set DiffAuxC from Tc
  double DauxCC=0.5*DauxC*tstep/(xstep*xstep);
  double DauxWW=0.5*DauxW*tstep/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall grid point: only diffuse towards other wall grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);

			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][0]=v*DauxWW;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);

			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  DiffAuxC[i][j][1]=v*DauxWW;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		DiffAuxC[i][j][2]=DauxWW;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		DiffAuxC[i][j][3]=DauxWW;
	  }
	else if(Tc[i][j]==1 || Tc[i][j]==2)//cell grid point: only diffuse towards other cell grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==1 || Tc[i+1][j]==2)
		{
#ifdef VOLUME3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
			  
			  /*
			   if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][0]=v*DauxCC;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==1 || Tc[i-1][j]==2)
		{
#ifdef VOLUME3D
		  if(i>xm)//right half, than your radius bigger than in i-1
		    {
		      
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
			  
			  /*
			  if(j<yLRCstraight)
			    vf=0.98;
			  else
			    vf=1;
			  */

			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;  
#endif
		  DiffAuxC[i][j][1]=v*DauxCC;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==1 || Tc[i][j+1]==2)
		DiffAuxC[i][j][2]=DauxCC;
	    if(j!=0)
	      if(Tc[i][j-1]==1 || Tc[i][j-1]==2)
		DiffAuxC[i][j][3]=DauxCC;
	  }
      }

//Now we need to adjust for getting a normal influx/efflux relation between root and shoot
#ifdef NORMALSHOOTROOTCONNECTION
  //double auxinvirtlayer=1.5*0.8*0.5*0.8;//defaultvalue
  double auxinvirtlayer=1.5*0.8*0.5*0.8*1*2;//1;//0.5;//10;//0.5//XXX
  //if(t>250000)
  //auxinvirtlayer*=1.2;
#endif
#ifdef SHOOTROOTCUT
  double auxinvirtlayer=0.8*0.5*exp(-0.003*time);//0;
#endif
  int jbot;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {

	if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	  {
	    c=CellGrid[i][j-1];
	    if(Tct[i][j-1]==5 || Tct[i][j-1]==6 || Tct[i][j-1]==7)//efflux to shoot from epi/cort/endo
	      {
		jbot=c->CellMinJ-1;
#ifndef VOLUME3D
		daux[i][j]=-0.1*Inf[i][jbot][2];
		//daux[i][j]=-0.1*(inf*2.5)*invspstep;//
		//daux[i][j]=-15*pPUMP*invspstep;//
#else
		daux[i][j]=-0.1*0.5*Inf[i][jbot][2];
#endif
		//daux[i][j]=-0.1*5*aux1fac*invspstep;
	      }
	    else if(Tct[i][j-1]==8 || Tct[i][j-1]==9 || Tct[i][j-1]==10 || Tct[i][j-1]==11)//influx from shoot to pericycle and vasculature
	      {
		jbot=c->CellMinJ-1;
		paux[i][j]=(Eff[i][jbot][2])*auxinvirtlayer;
		//paux[i][j]=5*pPUMP*invspstep*auxinvirtlayer;
	      }
	    break;
	  }
      }

}
