#include "Tissue.hh"

#include "BCell.hh"

void Tissue::Tdsolvetridiag(int ni)
{
  
  /**
   * ni - number of equations
   * a - sub-diagonal (means it is the diagonal below the main diagonal) -- indexed from 1..n-1
   * b - the main diagonal
   * c - sup-diagonal (means it is the diagonal above the main diagonal) -- indexed from 0..n
   * rhs - right part
   * x - the answer
   */
  int i;
  double mm;
  
  for (i = 1; i < ni; i++)
    {
      mm = a[i]/b[i-1];
      b[i] = b[i] - mm*c[i-1];
      rhs[i] = rhs[i] - mm*rhs[i-1];
    }

  res[ni-1] = rhs[ni-1]/b[ni-1];
  
  for (i = ni - 2; i >= 0; i--)
    res[i]=(rhs[i]-c[i]*res[i+1])/b[i];
  
  for(i=0;i<ni;i++)
    rhs[i]=res[i];

}


void Tissue::UpdateAuxinTissue()
{
  int i,j;
  static int Direction=0;
  int q;
  double func;
  double Sat=250;//efflux so PINs and baseline
  double Sat2=125;//125;//250;//1000;//influx, so AUX/LAX, but not passive influx
  double invspstep=1.0/(double)(xstep);

#ifdef AUXINTHROUGHPLASMODESMATA
  //auxin flux through plasmodesmata
  //as cells age pd get less open, especially outside vasc  
  double pdfauxflux=DauxC/1000;
  double temp=tstep*pdfauxflux/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    
	    if(Tc[i][j-1]==0 )//wall below, so basal membrane 
	      {
		int jj=j-1;
		
		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			if(j>yQC+3*HMZ  && jj>yQC+3*HMZ)
			  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i][jj]-Caux1[i][j]);
			else
			  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i][jj]-Caux1[i][j]);
			break;
		      }		 
		  }
	      }
	  else if(Tc[i][j+1]==0)//wall above, so apical membrane
	      {
		int jj=j+1;

		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			if(j>yQC+3*HMZ  && jj>yQC+3*HMZ)
			  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i][jj]-Caux1[i][j]);
			else
			  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i][jj]-Caux1[i][j]);
			break;
		      }		 
		  }
	      }


	  if(Tc[i-1][j]==0 )//wall to left, so left membrane
	    {
	      int ii=i-1;
	      while(ii>0 && (i-ii)<4)
		{
		  ii--;
		  if(Tc[ii][j]==2)//neighbouring membrane found
		    {
		      if(j>yQC+3*HMZ)
			Caux2[i][j]=Caux1[i][j]+temp*(Caux1[ii][j]-Caux1[i][j]);
		      else
			Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[ii][j]-Caux1[i][j]);
		      break;
		    }
		}
	    }
	  else if(Tc[i+1][j]==0 )//wall to right, so right membrane
	    {
	      int ii=i+1;
	      while(ii<n && (ii-i)<4)
		{
		  ii++;
		  if(Tc[ii][j]==2)//neighbouring membrane found
		    {
		      if(j>yQC+3*HMZ)
			Caux2[i][j]=Caux1[i][j]+temp*(Caux1[ii][j]-Caux1[i][j]);
		      else
			Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[ii][j]-Caux1[i][j]);
		      break;
		    }
		}
	    }
	}
    }
  
    
  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tc[i+2][j]==2)//membrane to the right
	      {
		if(j>yQC+3*HMZ)
		  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i+2][j]-Caux1[i][j]);
		else
		  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i+2][j]-Caux1[i][j]);
		      
	      }
	    else if(Tc[i-2][j]==2)//membrane to the left
	      {
		if(j>yQC+3*HMZ)
		  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i-2][j]-Caux1[i][j]);
		else
		  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i-2][j]-Caux1[i][j]);
	      }
	    if(Tc[i][j+2]==2)//membrane up
	      {
		if(j>yQC+3*HMZ  && (j+2)>yQC+3*HMZ)
		  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i][j+2]-Caux1[i][j]);
		else
		  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i][j+2]-Caux1[i][j]);
	      }
	    else if(Tc[i][j-2]==2)//membrane down
	      {
		if(j>yQC+3*HMZ && (j-2)>yQC+3*HMZ)
		  Caux2[i][j]=Caux1[i][j]+temp*(Caux1[i][j-2]-Caux1[i][j]);
		else
		  Caux2[i][j]=Caux1[i][j]+0.1*temp*(Caux1[i][j-2]-Caux1[i][j]);
	      }
	  }
      }
  */


  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      } 
#endif


  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction

#ifdef SATINFLUX
      //printf("satinflux\n");
      //Set Inf based on grid identities: is there flux?, in which direction?
      for(i=0;i<n;i++)
	for(j=0;j<m;j++)
	  {
	    if(Tc[i][j]==0)
	      {
		//nearest cell:
		if(i!=0)
		  if(Tc[i-1][j]==2)
		    Inf[i][j][1]=-(inf+(aux1fac*Aux1M[i-1][j]/((Caux1[i][j]/Sat2)+1))+(aux1fac*Lax3M[i-1][j]/((Caux1[i][j]/Sat2)+1)))*invspstep;
		if(i!=n-1)	
		  if(Tc[i+1][j]==2)
		    Inf[i][j][0]=-(inf+(aux1fac*Aux1M[i+1][j]/((Caux1[i][j]/Sat2)+1))+(aux1fac*Lax3M[i+1][j]/((Caux1[i][j]/Sat2)+1)))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==2)
		    Inf[i][j][3]=-(inf+(aux1fac*Aux1M[i][j-1]/((Caux1[i][j]/Sat2)+1))+(aux1fac*Lax3M[i][j-1]/((Caux1[i][j]/Sat2)+1)))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==2)
		    Inf[i][j][2]=-(inf+(aux1fac*Aux1M[i][j+1]/((Caux1[i][j]/Sat2)+1))+(aux1fac*Lax3M[i][j+1]/((Caux1[i][j]/Sat2)+1)))*invspstep;
	      }
	    else if(Tc[i][j]==2)
	      {
		if(i!=n-1)
		  if(Tc[i+1][j]==0)
		    Inf[i][j][0]=(inf+(aux1fac*Aux1M[i][j]/((Caux1[i+1][j]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Caux1[i+1][j]/Sat2)+1)))*invspstep;
		if(i!=0)
		  if(Tc[i-1][j]==0)
		    Inf[i][j][1]=(inf+(aux1fac*Aux1M[i][j]/((Caux1[i-1][j]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Caux1[i-1][j]/Sat2)+1)))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==0)
		    Inf[i][j][2]=(inf+(aux1fac*Aux1M[i][j]/((Caux1[i][j+1]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Caux1[i][j+1]/Sat2)+1)))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==0)
		    Inf[i][j][3]=(inf+(aux1fac*Aux1M[i][j]/((Caux1[i][j-1]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Caux1[i][j-1]/Sat2)+1)))*invspstep;
	      }
	  }
#endif      
#ifdef SATEFFLUX
      for(i=0;i<n;i++)
	for(j=0;j<m;j++)
	  {
	    if(Tc[i][j]==0)
	      {
		//nearest cell:
		if(i!=0)
		  if(Tc[i-1][j]==2)
		    Eff[i][j][1]=(eff/((Caux1[i-1][j]/Sat)+1)+pPUMP*Cpin1M[i-1][j]/((Caux1[i-1][j]/Sat)+1))*invspstep;
		if(i!=n-1)	
		  if(Tc[i+1][j]==2)
		    Eff[i][j][0]=(eff/((Caux1[i+1][j]/Sat)+1)+pPUMP*Cpin1M[i+1][j]/((Caux1[i+1][j]/Sat)+1))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==2)
		    Eff[i][j][3]=(eff/((Caux1[i][j-1]/Sat)+1)+pPUMP*Cpin1M[i][j-1]/((Caux1[i][j-1]/Sat)+1))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==2)
		    Eff[i][j][2]=(eff/((Caux1[i][j+1]/Sat)+1)+pPUMP*Cpin1M[i][j+1]/((Caux1[i][j+1]/Sat)+1))*invspstep;
	      }
	    else if(Tc[i][j]==2)
	      {
		if(i!=n-1)
		  if(Tc[i+1][j]==0)
		    Eff[i][j][0]=-(eff/((Caux1[i][j]/Sat)+1)+pPUMP*Cpin1M[i][j]/((Caux1[i][j]/Sat)+1))*invspstep;
		if(i!=0)
		  if(Tc[i-1][j]==0)
		    Eff[i][j][1]=-(eff/((Caux1[i][j]/Sat)+1)+pPUMP*Cpin1M[i][j]/((Caux1[i][j]/Sat)+1))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==0)
		    Eff[i][j][2]=-(eff/((Caux1[i][j]/Sat)+1)+pPUMP*Cpin1M[i][j]/((Caux1[i][j]/Sat)+1))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==0)
		    Eff[i][j][3]=-(eff/((Caux1[i][j]/Sat)+1)+pPUMP*Cpin1M[i][j]/((Caux1[i][j]/Sat)+1))*invspstep;
	      }
	  }
#endif   

      Direction=1;    
      //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else   
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1 
		  b[j]=1-(a[j]+c[j]);//j 
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }
		  
		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]); 
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  Tdsolvetridiag(m); 
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
    }
  if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction

#ifdef SATINFLUX
      //printf("satinflux\n");
      //Set Inf based on grid identities: is there flux?, in which direction?
      for(i=0;i<n;i++)
	for(j=0;j<m;j++)
	  {
	    if(Tc[i][j]==0)
	      {
		//nearest cell:
		if(i!=0)
		  if(Tc[i-1][j]==2)
		    Inf[i][j][1]=-(inf+(aux1fac*Aux1M[i-1][j]/((Cauxint[i][j]/Sat2)+1))+(aux1fac*Lax3M[i-1][j]/((Cauxint[i][j]/Sat2)+1)))*invspstep;
		if(i!=n-1)	
		  if(Tc[i+1][j]==2)
		    Inf[i][j][0]=-(inf+(aux1fac*Aux1M[i+1][j]/((Cauxint[i][j]/Sat2)+1))+(aux1fac*Lax3M[i+1][j]/((Cauxint[i][j]/Sat2)+1)))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==2)
		    Inf[i][j][3]=-(inf+(aux1fac*Aux1M[i][j-1]/((Cauxint[i][j]/Sat2)+1))+(aux1fac*Lax3M[i][j-1]/((Cauxint[i][j]/Sat2)+1)))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==2)
		    Inf[i][j][2]=-(inf+(aux1fac*Aux1M[i][j+1]/((Cauxint[i][j]/Sat2)+1))+(aux1fac*Lax3M[i][j+1]/((Cauxint[i][j]/Sat2)+1)))*invspstep;
	      }
	    else if(Tc[i][j]==2)
	      {
		if(i!=n-1)
		  if(Tc[i+1][j]==0)
		    Inf[i][j][0]=(inf+(aux1fac*Aux1M[i][j]/((Cauxint[i+1][j]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Cauxint[i+1][j]/Sat2)+1)))*invspstep;
		if(i!=0)
		  if(Tc[i-1][j]==0)
		    Inf[i][j][1]=(inf+(aux1fac*Aux1M[i][j]/((Cauxint[i-1][j]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Cauxint[i-1][j]/Sat2)+1)))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==0)
		    Inf[i][j][2]=(inf+(aux1fac*Aux1M[i][j]/((Cauxint[i][j+1]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Cauxint[i][j+1]/Sat2)+1)))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==0)
		    Inf[i][j][3]=(inf+(aux1fac*Aux1M[i][j]/((Cauxint[i][j-1]/Sat2)+1))+(aux1fac*Lax3M[i][j]/((Cauxint[i][j-1]/Sat2)+1)))*invspstep;
	      }
	  }
#endif      
#ifdef SATEFFLUX
      for(i=0;i<n;i++)
	for(j=0;j<m;j++)
	  {
	    if(Tc[i][j]==0)
	      {
		//nearest cell:
		if(i!=0)
		  if(Tc[i-1][j]==2)
		    Eff[i][j][1]=(eff/((Cauxint[i-1][j]/Sat)+1)+pPUMP*Cpin1M[i-1][j]/((Cauxint[i-1][j]/Sat)+1))*invspstep;
		if(i!=n-1)	
		  if(Tc[i+1][j]==2)
		    Eff[i][j][0]=(eff/((Cauxint[i+1][j]/Sat)+1)+pPUMP*Cpin1M[i+1][j]/((Cauxint[i+1][j]/Sat)+1))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==2)
		    Eff[i][j][3]=(eff/((Cauxint[i][j-1]/Sat)+1)+pPUMP*Cpin1M[i][j-1]/((Cauxint[i][j-1]/Sat)+1))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==2)
		    Eff[i][j][2]=(eff/((Cauxint[i][j+1]/Sat)+1)+pPUMP*Cpin1M[i][j+1]/((Cauxint[i][j+1]/Sat)+1))*invspstep;
	      }
	    else if(Tc[i][j]==2)
	      {
		if(i!=n-1)
		  if(Tc[i+1][j]==0)
		    Eff[i][j][0]=-(eff/(Cauxint[i][j]/Sat+1)+pPUMP*Cpin1M[i][j]/((Cauxint[i][j]/Sat)+1))*invspstep;
		if(i!=0)
		  if(Tc[i-1][j]==0)
		    Eff[i][j][1]=-(eff/(Cauxint[i][j]/Sat+1)+pPUMP*Cpin1M[i][j]/((Cauxint[i][j]/Sat)+1))*invspstep;
		if(j!=m-1)
		  if(Tc[i][j+1]==0)
		    Eff[i][j][2]=-(eff/(Cauxint[i][j]/Sat+1)+pPUMP*Cpin1M[i][j]/((Cauxint[i][j]/Sat)+1))*invspstep;
		if(j!=0)
		  if(Tc[i][j-1]==0)
		    Eff[i][j][3]=-(eff/(Cauxint[i][j]/Sat+1)+pPUMP*Cpin1M[i][j]/((Cauxint[i][j]/Sat)+1))*invspstep;
	      }
	  }
#endif   

      Direction=0;
      
      //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }
		  
		  
		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  Tdsolvetridiag(n); 
	  for(i=0;i<n;i++)
	    { 
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  iter it;
  BCell *c;
  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  (*it)->MeanCells=0;
	  it++;
	}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)
	  {
	    c=CellGrid[i][j];
	    c->MeanCells+=Caux1[i][j];
	  }
      }
  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  if((*it)->CellSize>0)
	    (*it)->MeanCells=(*it)->MeanCells*(*it)->InvCellSize;
	  it++;
	}
    }
 
  //final update for next timestep
  //doe dit pas na het berekenen van het 
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.    
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }
  

#ifdef PERI3D
  int leftepiout=disttoepi;
  int leftcortout=disttoepi+totsizeepi;
  int leftendoout=disttoepi+totsizeepi+totsizecort;
  int rightepiout=n-1-leftepiout;
  int rightcortout=n-1-leftcortout;
  int rightendoout=n-1-leftendoout;
  int leftperiout=disttoepi+totsizeepi+totsizecort+totsizeendo;
  int leftperiin=leftperiout+totsizeperi;
  int rightperiin=n-1-leftperiin;
  int rightperiout=n-1-leftperiin;
  double perc;
  double percbase=0.05*16./46.0;
  j=yLRCstraight;
  i=xm-2;
  while(Tc[i][j]!=3)
    {
      
      perc=((double)(xm-leftepiout)/(double)(xm-leftepiout))*percbase;
      Caux1[leftepiout][j]=Caux1[leftepiout][j]-perc*Caux1[leftepiout][j]+0.5*(perc*(Caux1[leftepiout][j]+Caux1[rightepiout][j]));
      Caux1[rightepiout][j]=Caux1[rightepiout][j]-perc*Caux1[rightepiout][j]+0.5*(perc*(Caux1[leftepiout][j]+Caux1[rightepiout][j]));

      perc=((double)(xm-leftepiout)/(double)(xm-leftcortout))*percbase;
      Caux1[leftcortout][j]=Caux1[leftcortout][j]-perc*Caux1[leftcortout][j]+0.5*(perc*(Caux1[leftcortout][j]+Caux1[rightcortout][j]));
      Caux1[rightcortout][j]=Caux1[rightcortout][j]-perc*Caux1[rightcortout][j]+0.5*(perc*(Caux1[leftcortout][j]+Caux1[rightcortout][j]));

      perc=((double)(xm-leftepiout)/(double)(xm-leftendoout))*percbase;
      Caux1[leftendoout][j]=Caux1[leftendoout][j]-perc*Caux1[leftendoout][j]+0.5*(perc*(Caux1[leftendoout][j]+Caux1[rightendoout][j]));
      Caux1[rightendoout][j]=Caux1[rightendoout][j]-perc*Caux1[rightendoout][j]+0.5*(perc*(Caux1[leftendoout][j]+Caux1[rightendoout][j]));

      perc=2*((double)(xm-leftepiout)/(double)(xm-leftperiout))*percbase;
      Caux1[leftperiout][j]=Caux1[leftperiout][j]-perc*Caux1[leftperiout][j]+0.5*(perc*(Caux1[leftperiout][j]+Caux1[rightperiout][j]));
      Caux1[rightperiout][j]=Caux1[rightperiout][j]-perc*Caux1[rightperiout][j]+0.5*(perc*(Caux1[leftperiout][j]+Caux1[rightperiout][j]));

      
      perc=2*((double)(xm-leftepiout)/(double)(xm-leftperiin))*percbase;
      Caux1[leftperiin][j]=Caux1[leftperiin][j]-perc*Caux1[leftperiin][j]+0.5*(perc*(Caux1[leftperiin][j]+Caux1[rightperiin][j]));
      Caux1[rightperiin][j]=Caux1[rightperiin][j]-perc*Caux1[rightperiin][j]+0.5*(perc*(Caux1[leftperiin][j]+Caux1[rightperiin][j]));
      

      j++;
    }
#endif
  

  
  //#define VASC3D
#ifdef VASC3D
  //do artificial semi-3D thing:
  //make part of auxin from inside wall endodermis
  //not go to pericycle but go to middle of vasculature
  i=disttoepi+totsizeepi+totsizecort+totsizeendo;
  int ii=n-1-i;
  j=yLRCstraight;
 

  double temp1,temp2;
  while(Tc[i][j]!=3)//for(j=yLRCstraight;j<m;j++)
    {
      temp1=tstep*0.25*Caux1[i][j];//0.75
      temp2=tstep*0.25*Caux1[ii][j];//0.5
       Caux1[i][j]-=temp1;
       Caux1[ii][j]-=temp2;
       Caux1[xm][j]+=temp1+temp2;
       j++;
    }
#endif
  
}

void Tissue::UpdateGeneExpressionTissue(int t)
{
  iter it;
  BCell *c;
  BCell *cdown;
  BCell *cdown2;
  BCell *cup;
  BCell *cup2;
  double kmahp6=40;//lower than km of CK?
  double kmckl=40;
  double kms1=35;
  double kms2=35;
  double kmdif=2.5;//15;
 
  double threshold=0.1;

  double ckself;
  double ckup,ckup2;
  double ckdown,ckdown2;
  int lengthtotal;


  double prodahp6=10*0.1;
  double degrahp6=10*0.001;
  double prodck=10*0.1;
  double degrck=10*0.001;
  double prodcksignal=10*0.1;
  double degrcksignal=10*0.001;
  static double tspeedupstep=tstep*speedup2;
  //possible issues with polarisation of pericycle:
  //1. due to cell size increase general tendency of auxin increase when moving shootward
  //try now to avoid it by starting with polarisation not at start of EZ but probably still not okay
  //2. if pericycle does not pump down any more possibly lot of auxin loss for root tip
  //try now to avoid it by have define 3DVASCULATURE on and having 0.25 of inner endodermis wall
  //auxin pumped to the middle of the vasculature
  //3. numerical issues, auxin seems to go to plus/minus infinity upon PIN repolarisation
  

  //actual gene expression
  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  if((*it)->CellSize>0)
	    {
	      (*it)->UpdateCell(t);
#ifdef AHP6SIGNALLING
	      c=(*it);
	     
	      if(Tct[c->CellMinI][c->CellMinJ]==8 )//perhaps replace by Diff th
		{

		  if(c->Diff>75)//if(c->Diff>80)//c->CellMinJ>yLRC1+25*HMZ)
		    {

#ifdef NUEEVENDEZENIET
		      c->dAHP6=0;
		      c->dCKlevel=0;
		      c->dCKmean=0;
		      c->dCKsignal=0;

		      //c->dAHP6=tspeedupstep*(prodahp6*((c->AuxinSign*c->AuxinSign/(c->AuxinSign*c->AuxinSign+kmahp6*kmahp6))*(50*50/(c->CKsignal*c->CKsignal+50*50)))-degrahp6*c->AHP6);
		      c->dAHP6=tspeedupstep*(prodahp6*((c->AuxinSign*c->AuxinSign/(c->AuxinSign*c->AuxinSign+kmahp6*kmahp6)))-degrahp6*c->AHP6);
		      c->dCKlevel=tspeedupstep*(prodck*(c->AuxinSign*c->AuxinSign/(c->AuxinSign*c->AuxinSign+kmckl*kmckl))-degrck*c->CKlevel);
		      
		      //does size weighting cause trouble? i.e. primed cell instead of one below polarizing?
		      c->CKmean=c->CKlevel*c->CellHeight;
		      lengthtotal=c->CellHeight;
		      cdown=CellGrid[c->CellMinI][c->CellMinJ-2];//lower nb
		      c->CKmean+=cdown->CKlevel*cdown->CellHeight;
		      lengthtotal+=cdown->CellHeight;
		      cdown2=CellGrid[cdown->CellMinI][cdown->CellMinJ-2];
		      c->CKmean+=cdown2->CKlevel*cdown2->CellHeight;
		      lengthtotal+=cdown2->CellHeight;
		      cup=CellGrid[c->CellMinI][c->CellMinJ+2];//upper nb
		      if(cup!=NULL)
			{
			  c->CKmean+=cup->CKlevel*cup->CellHeight;
			  lengthtotal+=cup->CellHeight;
			  cup2=CellGrid[cup->CellMinI][cup->CellMinJ+2];//upper nb
			  if(cup2!=NULL)
			    {
			      c->CKmean+=cup2->CKlevel*cup2->CellHeight;
			      lengthtotal+=cup2->CellHeight;
			    }
			}
		      c->CKmean/=lengthtotal;//based on CK levels earlier timestep
		      
		      c->dCKsignal=tspeedupstep*(prodcksignal*(c->CKmean*c->CKmean/(c->CKmean*c->CKmean+c->AHP6*c->AHP6 + 35*35))-degrcksignal*c->CKsignal);
		      //c->dCKsignal=tspeedupstep*(prodcksignal*((c->CKmean*c->CKmean/(c->CKmean*c->CKmean+ 35*35))*(50*50/(c->AHP6*c->AHP6)))-degrcksignal*c->CKsignal);
#endif

#define NUEVENNIET
#ifdef NUEEVENNIET
		      //AHP6 as instantaneous function of own auxin level
		      c->AHP6=100*c->AuxinSign*c->AuxinSign/(c->AuxinSign*c->AuxinSign+kmahp6*kmahp6);
		      //CK as instantaneous function of own auxin level
		      c->CKlevel=100*c->AuxinSign*c->AuxinSign/(c->AuxinSign*c->AuxinSign+kmckl*kmckl);
		      //averaged CK level, to mimick fast diffusion of CK
		      //note that for uppermost cell there could be no cell above it
		      ckself=c->CKlevel;
		      lengthtotal=c->CellHeight;
		      cdown=CellGrid[c->CellMinI][c->CellMinJ-2];//lower nb
		      ckdown=100*cdown->AuxinSign*cdown->AuxinSign/(cdown->AuxinSign*cdown->AuxinSign+kmckl*kmckl);
		      lengthtotal+=cdown->CellHeight;
		      cdown2=CellGrid[cdown->CellMinI][cdown->CellMinJ-2];
		      ckdown2=100*cdown2->AuxinSign*cdown2->AuxinSign/(cdown2->AuxinSign*cdown2->AuxinSign+kmckl*kmckl);
		      lengthtotal+=cdown2->CellHeight;
		      cup=CellGrid[c->CellMinI][c->CellMinJ+2];//upper nb
		      if(cup!=NULL)
			{
			  ckup=100*cup->AuxinSign*cup->AuxinSign/(cup->AuxinSign*cup->AuxinSign+kmckl*kmckl);
			  lengthtotal+=cup->CellHeight;
			  cup2=CellGrid[cup->CellMinI][cup->CellMinJ+2];//upper nb
			  if(cup2!=NULL)
			    {
			      ckup2=100*cup2->AuxinSign*cup2->AuxinSign/(cup2->AuxinSign*cup2->AuxinSign+kmckl*kmckl);
			      lengthtotal+=cup2->CellHeight;
			      c->CKmean=(ckself*c->CellHeight+ckdown*cdown->CellHeight+ckdown2*cdown2->CellHeight+ckup*cup->CellHeight+ckup2*cup2->CellHeight)/lengthtotal;
			    }
			  else
			    c->CKmean=(ckself*c->CellHeight+ckdown*cdown->CellHeight+ckdown2*cdown2->CellHeight+ckup*cup->CellHeight)/lengthtotal;
			}
		      else
			{
			  c->CKmean=(ckself*c->CellHeight+ckdown*cdown->CellHeight+ckdown2*cdown2->CellHeight)/lengthtotal;
			}
		      //polarisation tendency
		      //ahp6 blocks CK signalling so determines receptor availability
		      //ck availability determines signal level received by available receptors
		      //c->CKsignal=100*(c->CKmean*c->CKmean/(c->CKmean*c->CKmean+1.5*1.5*c->AHP6*c->AHP6 + 35*35));
		      c->CKsignal=100*(c->CKmean*c->CKmean/(c->CKmean*c->CKmean+c->AHP6*c->AHP6 + 35*35));
		      
		      //polarisation direction and signal
		      if(cup!=NULL)
			c->auxindif=cup->AuxinSign-cdown->AuxinSign;//up - down
		      else
			c->auxindif=c->AuxinSign-cdown->AuxinSign;//if no upper cell
#endif		      
		    
		      
		      double alpha=cup->AuxinSign/(cup->AuxinSign+cdown->AuxinSign);
		      
		      c->pinup=0+alpha*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35));
		      c->pindown=1-alpha*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35));
		      c->pinin=min(0.1,0.1*(1-1.*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35))));
		      
		      
		      /*
			if(c->auxindif>0)//upper cell more auxin
			{
			c->polarisation=1;
			c->pinup=min(1.,1.*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35)));
			c->pindown=min(1,1-1.*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35)));
			c->pinin=min(0.1,0.1*(1-1.*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35))));
			
			//printf("cell %i gets pinup %f at time %i\n",c->cellid,c->pinup,t);
			}
			else//lower cell more auxinc //DOES THIS CAUSE THE UPWARD POLARITY LOSS??
			{
			c->auxindif=-c->auxindif;
			c->polarisation=2;
			c->pinup=0;
			c->pindown=max(1,1+0.1*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35)));
			c->pinin=min(0.1,0.1*(1-0.1*(c->CKsignal*c->CKsignal/(c->CKsignal*c->CKsignal+35*35))));
			}
		      */
		    }
		   
		  /*else//why do we need this, cells do not move back, and cells here should hence not be updated unless moving to higher zone?
		    {
		      c->polarisation=0;
		      c->pinup=0;
		      c->pindown=1;
		      c->pinin=0.1;
		      }*/
		    
		}//pericycle cell 
#endif
	    }//cell size larger than zero
	  it++;
	}//list not at end
    }//list not null


#ifdef NUEEVENDEZENIET
  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  if((*it)->CellSize>0)
	    {
	      (*it)->UpdateCell(t);
#ifdef AHP6SIGNALLING
	      c=(*it);
	     
	      if(Tct[c->CellMinI][c->CellMinJ]==8 )//perhaps replace by Diff th
		{

		  if(c->Diff>75)//if(c->Diff>80)//c->CellMinJ>yLRC1+25*HMZ)
		    {
		      c->AHP6+=c->dAHP6;
		      c->CKlevel+=c->dCKlevel;
		      c->CKsignal+=c->dCKsignal;
		    }
		}//pericycle cell 
#endif
	    }//cell size larger than zero
	  it++;
	}//list not at end
    }//list not null
#endif

  //flux of proteins through plasmodesmata
#ifdef PLASMODESMATAOPEN
  int i,j,ii,jj;
  double pdflux=4*0.01/(double)xstep;
  //initialise PLTs on grid
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//cell there
	  {
	    c=CellGrid[i][j];
	    Pltsgrid[i][j]=c->Plts;
	    Pltsgrid2[i][j]=c->Plts;
	  }
	else
	  {
	    Pltsgrid[i][j]=0;
	    Pltsgrid2[i][j]=0;
	  }
      }
 
  //PLTs fluxes between cells through plasmodesmata
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//mebrane, so point of possible exchange
	  {
	    if(Tc[i-1][j]==0)//wall to left, so left membrane
	      {
		ii=i-1;
		while(ii>0 && (i-ii)<4)
		  {
		    ii--;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i+1][j]==0)//wall to right, so right membrane
	      {
		ii=i+1;
		while(ii<n && (ii-i)<4)
		  {
		    ii++;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j-1]==0)//wall above, so apical membrane ???????
	      {
		jj=j-1;
		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	    if(Tc[i][j+1]==0)//wall below, so basal membrane ?????????
	      {
		jj=j+1;
		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	  }
      }
#ifdef PLTLEAKAGETOMAINROOT
  double Pltslevelshoot=0;
  double pdfluxshoot=pdflux;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	if(CellGrid[i][j]!=NULL)//topmost part of topmost cell found
	  {
	    Pltsgrid2[i][j]+=pdfluxshoot*(Pltslevelshoot-Pltsgrid[i][j]);
	    j=-100;//so break from second for loop
	  }
      }
#endif
  
    
  //transform back from grid to cellular
  //assumption that within cell PLTs differences
  //even out quickly through diffusion
  if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  (*it)->Plts=0;
	  it++;
	}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//cell there
	  {
	    c=CellGrid[i][j];
	    c->Plts+Pltsgrid2[i][j];
	  }
      }
   if(CellList!=NULL)
    {  
      it=CellList->begin();
      while(it!=CellList->end())
	{
	  
	  if((*it)->CellSize>0)
	    (*it)->Plts=(*it)->Plts*(*it)->InvCellSize;
	  it++;
	}
    } 
#endif
}


void Tissue::UpdateFluxesTissue(int t)
{
  BCell *c;
  BCell *cacross;
  iter it;
  int i,j,k,ii;
  double invspstep=1.0/(double)(xstep);

  BCell *cos;
  BCell *cout;
  BCell *cin;
  

  //growth -> location cells -> zones
  //zones -> transporter prepatterns
#ifndef DEBUGMODE
   for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	ii=n-1-i;
	PinPrePat[i][j]=0;
	PinPrePat[ii][j]=0;
	Aux1PrePat[i][j]=0;
	Aux1PrePat[ii][j]=0;
      }
#else
   for(i=0;i<n;i++)
     for(j=0;j<m;j++)
      {
	PinPrePat[i][j]=0;
	Aux1PrePat[i][j]=0;
      }
#endif

#ifndef RESTART
  if(t==0)
    {
      if(CellList!=NULL)
	{  
	  it=CellList->begin();
	  while(it!=CellList->end())
	    {
	      
	      if((*it)->CellMaxJ<yR)
		(*it)->CellZonationState=0;
	      else if((*it)->CellMaxJ<yLRC1-(15*HMZ+15))
		(*it)->CellZonationState=30;
	      else if((*it)->CellMaxJ<yLRC1)
		(*it)->CellZonationState=50;
	      else if((*it)->CellMaxJ<yLRC1+(7*HEZ+7))
		(*it)->CellZonationState=60;
	      else
		(*it)->CellZonationState=100;

	      it++;
	    }
	}
    }
#endif


#ifndef DEBUGMODE
  int kk,ll;
  int mindist;
  double disttresh;
  double distance; 
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    ii=n-1-i;
	    c=CellGrid[i][j];
	    cacross=CellGrid[ii][j];

	    if(Tct[i][j]<=4) //lateral root cap
	      {	
		if(Tmt[i][j]==1)//up
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		if(Tct[i][j]==1 )//Thea&& c->CellMinJ>yQC+10*HMZ+10)//uppermost lateral root cap
		  {
		    if(Tmt[i][j]==3)//inward membrane
		      {                                  	
			PinPrePat[i][j]=0.25;
			PinPrePat[ii][j]=0.25;
		      }
		  }  
		else //other lateral root cap cells
		  {
		    if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		      {
			PinPrePat[i][j]=0.25;
			PinPrePat[ii][j]=0.25;
		      } 
		  }
	      }
	    else if(Tct[i][j]==5)//epidermis 
	      {
		if(Tmt[i][j]==1)//up
		  {
		    if(j<yQC+10*HMZ+10)//low in MZ
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else if(c->CellZonationState<60)//untill MZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else//DZ
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		  {
		    if(Tmt[i][j]==3)//in
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }         	    
		    if(Tmt[i][j]==4 && c->CellMinJ<yQC+20*HMZ+20)//out to lrc
		      {
			PinPrePat[i][j]=0.3;
			PinPrePat[ii][j]=0.3;
		      }
		    else if(Tmt[i][j]==4 && c->CellZonationState<=50)//Thea?
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
	      }
	    else if(Tct[i][j]==6)//cortex 
	      {
		if(Tmt[i][j]==1 && c->CellZonationState>50)//up: beyond MZ
		  {
		    if(c->CellZonationState<60)//untill MZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else//DZ 
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		  }
		if(Tmt[i][j]==2 && c->CellZonationState<=50)//down: in MZ//FLIP AT TZ!!!
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
		else if(Tmt[i][j]==4 && c->CellZonationState>60)
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
		
	      }
	    else if(Tct[i][j]==7)//endodermis
	      {
		if(Tmt[i][j]==2)//down
		  {
		    if(c->CellZonationState<60)//till MZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.6;
			PinPrePat[ii][j]=0.6;
		      }
		    else
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		    else
		      {
			PinPrePat[i][j]=0.15;//0.25;
			PinPrePat[ii][j]=0.15;//0.25;
		      }
		  }
		else if(Tmt[i][j]==4 && c->CellZonationState==100)//out
		  {
		    PinPrePat[i][j]=0.15;
		    PinPrePat[ii][j]=0.15;
		  }
		else if(Tmt[i][j]==1 && c->CellZonationState==100)//up
		  {
		    PinPrePat[i][j]=0.15;
		    PinPrePat[ii][j]=0.15;
		  }
	      }
	    else if(Tct[i][j]==8)//pericycle 
	      {
		
		if(Tmt[i][j]==2)//down
		  {
		    if(c->CellZonationState<60)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.9;//0.8;//0.6;
			PinPrePat[ii][j]=0.9;//0.8;//0.6;
		      }
		    else
		      {
			PinPrePat[i][j]=0.7;//0.6;//0.35;
			PinPrePat[i][j]=0.7;//0.6;//0.35;
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  {
		    if(c->CellMinJ<yQC+5*HMZ+5)
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else if(Tmt[i][j]==3 )
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
	      }	      
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature 
	      {
		if(Tmt[i][j]==2)//down 
		  {
		    if(c->CellZonationState<60)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(c->CellZonationState<100)
		      {
			PinPrePat[i][j]=0.8;//0.6;
			PinPrePat[ii][j]=0.8;//0.6;
		      }
		    else
		      {
			PinPrePat[i][j]=0.6;//0.35;
			PinPrePat[ii][j]=0.6;//0.35;
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  {
		    if(c->CellMinJ<yQC+5*HMZ+5)
		      {
			PinPrePat[i][j]=0.4;
			PinPrePat[ii][j]=0.4;
		      }
		    else //if(Tmt[i][j]==4)//17 april 2019
		      {
			if(Tmt[i][j]==4)//7 mei
			  {
			    PinPrePat[i][j]=0.1;//0.15;//17 april 2019
			    PinPrePat[ii][j]=0.1;//0.15;//17 april 2019
			  }
			
		      //AHA, WE HANDLE ALL SYMMETRIC HERE, THIS
		      //CAN NOT WORK FOR SYMMETRY BREAKING, SO BELOW
		      //WE HANDLE THE I AND II CASE SEPEARATELY
		      
		      //UPREGULATING VASCULATURE PATTERN?
		      //do this based on auxin level (and/or memARF?) in
		      //(nearest or max) pericycle cell
		      //should change ratio of lateral versus downward
		      //transport
#ifdef VASCULARPOLARISATION
			if(c->CellMinJ>yLRC1)//if(c->CellMinJ>yLRC1+27*HMZ)// && Tct[i][j]==9)//EZ
			  {
			  int iout, jout, iin,jin;
			  double auxout,auxin;
			  double b, a, alpha;
			  int posperileft=disttoepi+totsizeepi+totsizecort+totsizeendo+2;
			  int posperiright=n-1-posperileft;

			  //i side is on left side of root
			  //for left side i-x is out i+x is in
			  iout=posperileft;//so peri own side
			  jout=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
			  cout=CellGrid[iout][jout];
			  while(cout==NULL)
			    {
			      jout--;
			      cout=CellGrid[iout][jout];
			    }
			  auxout=cout->AuxinSign;//*(1+0.001*cout->memARF);
			 
			  //ii side on right side of the root
			  iin=posperiright;//c->CellMaxI+2;//so peri other side
			  jin=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
			  cin=CellGrid[iin][jin];
			  while(cin==NULL)
			    {
			      jin--;
			      cin=CellGrid[iin][jin];
			    }
			  auxin=cin->AuxinSign;//*(1+0.001*cin->memARF);
			 
			 
			  double deltaconc=cout->AHP6-cin->AHP6;//auxout-auxin;
			  b=0.15;
			  a=0.1;
			  double kkmm=30;//15;
			  //if(abs(deltaconc)>1)//5)//something to do
			    {
			      //if(deltaconc>0)//auxout>auxin so left higher
				{
				  //do only something for i side
				  /*if(Tmt[i][j]==4)//out
				    PinPrePat[i][j]=b+a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  else
				  PinPrePat[i][j]=b-a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));*/
				  if(Tmt[i][j]==4)//out
				    PinPrePat[i][j]=b+a*(cout->AHP6*cout->AHP6/(cout->AHP6*cout->AHP6+kkmm*kkmm));
				  else
				    PinPrePat[i][j]=b-a*(cout->AHP6*cout->AHP6/(cout->AHP6*cout->AHP6+kkmm*kkmm));
				  //c->polarisation=4;

				  /*
				  if(Tmt[ii][j]==3)//in
				    PinPrePat[ii][j]=b+a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  else
				    PinPrePat[ii][j]=b-a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  c->polarisation=3;
				  */
				}
				//else//righthigher
				{
				  //do only something for ii side
				  deltaconc=-deltaconc;
				  /*if(Tmt[ii][j]==4)//out
				     PinPrePat[ii][j]=b+a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  else
				  PinPrePat[ii][j]=b-a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));*/
				  if(Tmt[ii][j]==4)//out
				     PinPrePat[ii][j]=b+a*(cin->AHP6*cin->AHP6/(cin->AHP6*cin->AHP6+kkmm*kkmm));
				  else
				    PinPrePat[ii][j]=b-a*(cin->AHP6*cin->AHP6/(cin->AHP6*cin->AHP6+kkmm*kkmm));
				  //c->polarisation=4;

				  /*
				  if(Tmt[i][j]==3)//in
				    PinPrePat[i][j]=b+a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  else
				    PinPrePat[i][j]=b-a*(deltaconc*deltaconc/(deltaconc*deltaconc+kkmm*kkmm));
				  c->polarisation=3;
				  */
				}
			    }
			    //else
			    //c->polarisation=0;
			}	
#endif
		      //POLARISING VASCULATURE PATTERN
		      //below is not very effcient as we compute this
		      //per vasculare lateral membrane grid point yet could 
		      //basically do it on a per vasc cell level
		      //also polarisation can change instantaneously as
		      //it is a direct function of auxin, perhaps better
		      //to make this a cell property with a differential
		      //equation and a timeconstant of its own
		      /*
		      if(c->CellMinJ>yLRC1+5*HEZ+5)// && Tct[i][j]==9)//EZ
			{
			  int iout, jout, iin,jin;
			  double auxout,auxin;
			  double b, a, alpha;
			  int posperileft=disttoepi+totsizeepi+totsizecort+totsizeendo+2;
			  int posperiright=n-1-posperileft;

			  
			  b=0.05;//0.1;
			  a=0.2;//0.4;

			  //i side is on left side of root
			  //for left side i-x is out i+x is in
			  iout=posperileft;//c->CellMinI-2;//so peri own side
			  jout=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
			  cout=CellGrid[iout][jout];
			  while(cout==NULL)
			    {
			      jout--;
			      cout=CellGrid[iout][jout];
			    }
			  auxout=cout->MeanCells;
			  iin=posperiright;//c->CellMaxI+2;//so peri other side
			  jin=(int)(0.5*(c->CellMinJ+c->CellMaxJ));
			  cin=CellGrid[iin][jin];
			  while(cin==NULL)
			    {
			      jin--;
			      cin=CellGrid[iin][jin];
			    }
			  auxin=cin->MeanCells;
			  // if(0.5*(auxin+auxout)>20 &&
			  //(auxin/auxout<0.9 || auxin/auxout>1.1))//enough auxin and 10% difference for polarisation
			  if((auxin>30 || auxout>30) && (auxin/auxout<0.95 || auxin/auxout>1.05)) 
			    alpha=auxin/(auxin+auxout);//so auxin peri other/sum peri auxin
			  else
			    alpha=0.5;
			  if(Tmt[i][j]==3)//inward membrane // so pointing to other peri
			    PinPrePat[i][j]=b+a*alpha;
			  else//outward membrane //so pointing to own peri
			    PinPrePat[i][j]=b+a*(1-alpha);

			  //ii side is on right side of root
			  //for right side i-x is in i+x is out
			  cos=CellGrid[ii][j];//cell at the other side ii 
			  iout=posperiright;//cos->CellMaxI+2;//so peri on own side
			  jout=(int)(0.5*(cos->CellMinJ+cos->CellMaxJ));
			  cout=CellGrid[iout][jout];
			  while(cout==NULL)
			    {
			      jout--;
			      cout=CellGrid[iout][jout];
			    }
			  auxout=cout->MeanCells;
			  iin=posperileft;//cos->CellMinI-2;//so peri on other side
			  jin=(int)(0.5*(cos->CellMinJ+cos->CellMaxJ));
			  cin=CellGrid[iin][jin];
			  while(cin==NULL)
			    {
			      jin--;
			      cin=CellGrid[iin][jin];
			    }
			  auxin=cin->MeanCells;
			  //if(0.5*(auxin+auxout)>20 &&
			  //(auxin/auxout<0.9 || auxin/auxout>1.1))//enough auxin and 10% difference for polarisation
			  if((auxin>30 || auxout>30) && (auxin/auxout<0.95 || auxin/auxout>1.05) ) 
			    alpha=auxin/(auxin+auxout);//so auxin peri other/sum peri auxin
			  else
			    alpha=0.5;
			  if(Tmt[ii][j]==3)//inward membrane
			    PinPrePat[ii][j]=b+a*alpha;
			  else//outwad membrane
			    PinPrePat[ii][j]=b+a*(1-alpha);
			    
			}
		      else
			{
			  if(Tmt[i][j]==4)
			    {
			      PinPrePat[i][j]=0.15;//0.3;
			      PinPrePat[ii][j]=0.15;//0.3
			    }
			}
		      */
		    }
		}
	      }
	    else if(Tct[i][j]==12)//QC
	      {
		if(j==c->CellMinJ)//works better than Tmt[i][j]==4
		  {
		    PinPrePat[i][j]=0.5;//2;//0.5;
		    PinPrePat[ii][j]=0.5;//2;//0.5;
		  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer
	      {
	
#ifdef FASTPIN3COL
		if(Tmt[i][j]<4)
		  {  
		    PinPrePat[i][j]=10*0.5;
		    PinPrePat[ii][j]=10*0.5;  
		  }
#else
#ifdef FASTPIN3COLDOWN
		if(Tmt[i][j]==4)
		  {
		    PinPrePat[i][j]=10*0.5;
		    PinPrePat[ii][j]=10*0.5;
		  }
		else
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
#else
		if(Tmt[i][j]<4)
		  {   
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
#endif
#endif
	      }
	    else if(Tct[i][j]>13)//columella except bottom layer
	      {
		if(Tct[i][j]<17)//not the topmost one
		  {
#ifdef FASTPIN3COL
		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=10*0.75;
			PinPrePat[ii][j]=10*0.75;
		      }
		    else 
		      {
			PinPrePat[i][j]=10*1;
			PinPrePat[ii][j]=10*1;
		      }
#else
#ifdef FASTPIN3COLDOWN
		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else//4
		      {
			PinPrePat[i][j]=10*1;
			PinPrePat[ii][j]=10*1;
		      }
#else

		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else 
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
#endif		    
#endif
		  }
		else
		  {

#ifdef FASTPIN3COL		    
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=10*0.75;
			PinPrePat[ii][j]=10*0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=10*1;
			PinPrePat[ii][j]=10*1;
		      }
#else
#ifdef FASTPIN3COLDOWN
		    if(Tmt[i][j]==4)
		      {
			PinPrePat[i][j]=10*1;
			PinPrePat[ii][j]=10*1;
		      }
		    else if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }

#else		    
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
#endif
#endif
		  }
	      }	   
	  }
      }
 
#ifdef AHP6SIGNALLING
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]==8)//pericycle 
	      {
		c=CellGrid[i][j];
		if(c->CKsignal>5 && c->auxindif>0)//WHAT HAPPENS NOW, WORSE OR BETTER?
		  {
		    if(Tmt[i][j]==1)
		      PinPrePat[i][j]=c->pinup;
		    else if(Tmt[i][j]==2)
		      PinPrePat[i][j]=c->pindown;
		    else if(Tmt[i][j]==3)
		      PinPrePat[i][j]=c->pinin;
		  }
	      }
	  }
      }
#endif

#ifdef  ABCB21
   for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]==8)//pericycle 
	      {
		ii=n-1-i;
		c=CellGrid[i][j];
		cacross=CellGrid[ii][j];
		//potentially asymmetric
		//ABCB21 exports auxin for higher auxin concentrations
		if(c->CellMinJ>yLRC1)
		  {
		    PinPrePat[i][j]+=0.3*(c->MeanCells*c->MeanCells/(c->MeanCells*c->MeanCells+50*50));
		    PinPrePat[ii][j]+=0.3*(cacross->MeanCells*cacross->MeanCells/(cacross->MeanCells*cacross->MeanCells+50*50));
		  }
	      }
	  }
      }
#endif


  BCell *cc;
#ifdef CYTOKININ
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)
	  {
	    cc=CellGrid[i][j];
	    if(cc->CellZonationState>=60)
	      PinPrePat[i][j]*=0.65;
	  }
      }
#endif

#ifdef LESSPIN2
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tct[i][j]==1 || Tct[i][j]==5 || Tct[i][j]==6)//lrc1, epi, cortex
	  {
	    if(Tmt[i][j]==1)
	      PinPrePat[i][j]*=0.75;
	  }
      }
#endif

#ifdef PINNECK
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    if(Tmt[i][j]==2 && (Tct[i][j]==5 || Tct[i][j]==6))//lower membrane epi, cortex
	      {
		c=CellGrid[i][j];
		if(c->CellMinJ>yLRC1 && c->CellMinJ<yEND-3*HEZ-5)//if(c->CellMinJ>yLRC1+4*HEZ+4)//if(c->CellMinJ>yLRC1)
		  {
		    PinPrePat[i][j]=1;
		  }
	      }
	  }
      }
#endif

  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    if(Tct[i][j]>=7 && Tct[i][j]<=11)
	    //if(Tct[i][j]!=5 && Tct[i][j]!=6)//not cort epi
	      {
		PinPrePat[i][j]*=0.2;
	      }
	  }
      }
  */
      
  //AUX1
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    ii=n-1-i;
	    c=CellGrid[i][j];

	    if(Tct[i][j]>=12 && Tct[i][j]!=13)//QC+nb columella layer
	      {
#ifndef LESSAUX1INCOL
		Aux1PrePat[i][j]=3;
#else
		Aux1PrePat[i][j]=1.5;
#endif
	      }
	    else if(Tct[i][j]==13 || (Tct[i][j]>=1 && Tct[i][j]<=4))//lowest columella layer + lrc1-4
	      {
		Aux1PrePat[i][j]=1; 

		/*
#ifdef NRT1DOT1AUXINTRANSPORT
		if(Tct[i][j]==13 || Tct[i][j]==1)
		  {
		    if(Tmt[i][j]==1 || Tmt[i][j]==2)
		      {
			if(c->CellMinJ<yLRC1-10*HMZ-10)
			  Aux1PrePat[i][j]+=1;//2;//1;
		      }
		  }
#endif
		*/
	      }
	    else if(Tct[i][j]==5)//epi
	      {

	
		if(c->CellMaxJ<yMZEZ-4*HMZ-4)//yMZEZ-12*HMZ-12)
		  Aux1PrePat[i][j]=0.1;
		else if(c->CellMaxJ<yMZEZ)//yMZEZ-4*HMZ-4)
		    Aux1PrePat[i][j]=0.6;
		else
#ifndef LESSAUX1INEZ
		  Aux1PrePat[i][j]=1;//1.5;//1;//LAX3
#else
		  Aux1PrePat[i][j]=0.6;
#endif
		
		
		  //Aux1PrePat[i][j]=1;

		  /*
#ifdef NRT1DOT1AUXINTRANSPORT
		if(Tmt[i][j]==1 || Tmt[i][j]==2)//NRT1.1 auxin transport
		  {
		    if(c->CellMinJ>yLRC1-4*HMZ-4)
		      Aux1PrePat[i][j]+=1;//2;//1;
		  }
		  #endif
		  */
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		if(c->CellMaxJ<yMZEZ-4*HMZ-4)
		   Aux1PrePat[i][j]=0.1;
		else
		  Aux1PrePat[i][j]=0.5;

		/*
		if(c->CellMaxJ>yMZEZ)
		  Aux1PrePat[i][j]=0.75;//LAX3
		*/
	      }
	    else if(Tct[i][j]>=8 && Tct[i][j]<=11)//peri and vasc
	      {
		
		if(c->CellMaxJ<yQC+10*HMZ+10)
		  Aux1PrePat[i][j]=2;
		else
		  Aux1PrePat[i][j]=1;
		
		/*
		if(c->CellMaxJ>yMZEZ)
		  Aux1PrePat[i][j]=1.5;//LAX3
		*/
	      }
	  }
      }

  //AUX1 same everywhere
  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  Aux1PrePat[i][j]=0.75;
      }
  */
  
  //LAX3
  for(i=0;i<n;i++)
   for(j=0;j<m;j++)
     {
      
	 Lax3PrePat[i][j]=0;
#ifdef NRT1DOT1AUXINTRANSPORT
	 if(Tc[i][j]==2)
	   {
	     c=CellGrid[i][j];
	     if(Tct[i][j]==5)//epi
	       {
		 if(c->CellMaxJ>=yMZEZ-4*HMZ-4)
		   {
		     if(Tmt[i][j]==1 || Tmt[i][j]==2)//NRT1.1 auxin transport
		       {
			   Lax3PrePat[i][j]+=1;//2;//1;
		       }
		   }
	       }
	     
	     if(Tct[i][j]==13 || Tct[i][j]==1)//lowest col, outer lrc
	       {
		 if(Tmt[i][j]==1 || Tmt[i][j]==2)
		   {
		     if(c->CellMinJ<yLRC1-10*HMZ-10)
		       Lax3PrePat[i][j]+=1;//2;//1;
		   }
	       }
	   }
#endif		
     }
#else//debug mode
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    Aux1PrePat[i][j]=1;
	    if(Tmt[i][j]==2)
	      PinPrePat[i][j]=1;
	  }
      }

#endif

  /*
  double dist;
  double scale;
  //3D scaling of strenght fluxes with radius
  //normalized to radius vasculature
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tmt[i][j]==3 || Tmt[i][j]==4)//lateral membrane
	  {
	    dist=abs(i-xm);
	    if(dist>(totsizeperi+totsizevasc))
	      {
		scale=((double)(dist))/((double)(totsizeperi+totsizevasc));
		PinPrePat[i][j]*=scale;
		Aux1PrePat[i][j]*=scale;
		Lax3PrePat[i][j]*=scale;
	      }
	  }
      }
  */

  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//if(Tmt[i][j]==2)//mebrane ah! membrane type not membrane!
	  {
	    if(Tct[i][j]==12)
	      PinPrePat[i][j]=2;//apolar and higher!// why not on all membranes: only membrane type 2, which in this part is nward
	    else if(Tct[i][j]>7 && Tct[i][j]<12)
	      {
		if(j<(yLRCstraight+5*HMZ))//if(j<yLRCstraight+5*HMZ)
		  {
		    if(Tmt[i][j]==3 || Tmt[i][j]==4)
		      {
			PinPrePat[i][j]=1;//also lateral ones 1//do not see this!!//indeed we do not get here
			//printf("point %i %i cell type %i membrane %i now also at 1\n",i,j,Tct[i][j],Tmt[i][j]);
		      }
		  }
	      }
	  }
      }
  */

  /*
  //so actually we did this in TestCheck2c
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2 && Tct[i][j]==12)
	  {
	    if(Tmt[i][j]==2)//inward membrane QC
	      PinPrePat[i][j]=2;//so more inward then to the bottom!
	  }
      }
  */
  /*
 for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]<5)//lrc 
	      PinPrePat[i][j]*=1.5;
	    else if(Tct[i][j]>11)//QC, columella
	      PinPrePat[i][j]*=3;
	    else if(Tct[i][j]>7 && Tct[i][j]<12)//lowest part vasculature
	      {
		if(j<yLRCstraight)
		  PinPrePat[i][j]*=1.5;
	      }
	  }
      }
  */

  



  //gene expression * prepatterns-> transporter membrane levels
  //update membrane protein concentration based on new prepatterns and gene expression levels
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(CellGrid[i][j]!=NULL)//grid point inside cell
	  {
	    c=CellGrid[i][j];
	    Cpin1M[i][j]=c->Pin*PinPrePat[i][j];
	    Aux1M[i][j]=c->Aux1*Aux1PrePat[i][j];	  
	    Lax3M[i][j]=c->Lax3*Lax3PrePat[i][j];
	  }
	else
	  {
	    Cpin1M[i][j]=0;
	    Aux1M[i][j]=0;
	    Lax3M[i][j]=0; 	    
	  }   	
      }
  //membrane levels -> fluxes
  //direction 0 i+1;
  //direction 1 i-1;
  //direction 2 j+1;
  //direction 3 j-1;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	daux[i][j]=0;//default no decay
	paux[i][j]=0;//default no production
	for(k=0;k<4;k++)
	  {	  
	    Inf[i][j][k]=0;
	    Eff[i][j][k]=0;
	    DiffAuxC[i][j][k]=0;
	  }
      }
  //daux depends on Gh33 gene expression
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	c=CellGrid[i][j];
	if(c!=NULL)
	  daux[i][j]=(1-gh33fac)*dAux+gh33fac*0.01*c->Gh33*dAux;
      }
#ifdef CYTOKININ
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	c=CellGrid[i][j];
	if(c!=NULL)
	  {
	    if(Tct[i][j]==1)
	      daux[i][j]=10*dAux;
	    else if(Tct[i][j]==5 && c->CellZonationState>=60)
	      daux[i][j]=10*dAux;
	  }
      }
#endif


  //paux depends on Yuc3 expression and cell type
#ifndef DEBUGMODE
  double volfactor;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
	{
	  ii=n-1-i;
	  c=CellGrid[i][j];
	  if(c!=NULL)
	    { 
	      if(Tct[i][j]==12 || Tct[i][j]==17)//QC and columella layer 5 
		volfactor=50*2;
	      else if(j<yQC+2*HMZ+2 && (Tct[i][j]>=8 && Tct[i][j]<=11))
		volfactor=50*2;
	      else if(Tct[i][j] > 12 && Tct[i][j]< 17 )//Columella layers 1-4 
		volfactor = 25*2;
	      else if(Tct[i][j]>=1 && Tct[i][j]<=4)//Lateral Root Cap
		volfactor=15*2;
	      else 
		volfactor=1;

#ifdef CYTOKININ
	      volfactor*=4;
#endif
	      //paux[i][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*c->Yuc3*pAux)*HMZ/c->CellHeight;
	      paux[i][j]=(volfactor*0.65*pAux+5*0.01*c->Yuc3*pAux)*HMZ/c->CellHeight;
	    }
	}
#else
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	 c=CellGrid[i][j];
	 if(c!=NULL)
	   {
	     paux[i][j]=((1-yuc3fac)*pAux+yuc3fac*0.01*c->Yuc3*pAux)*M1/c->CellHeight; 
	   }
      }
#endif
#ifdef AUXINAPPLICATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(i==0 || i==n-1 || j==0)
	  paux[i][j]+=1.00/xstep;
      }
#endif  
  //Set Inf from Lax3,Lax2,Aux1
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)
	  {
	    //nearest cell:
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		Inf[i][j][1]=(-inf-aux1fac*Aux1M[i-1][j]-aux1fac*Lax3M[i-1][j])*invspstep;
	    if(i!=n-1)	
	      if(Tc[i+1][j]==2)
		Inf[i][j][0]=(-inf-aux1fac*Aux1M[i+1][j]-aux1fac*Lax3M[i+1][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Inf[i][j][3]=(-inf-aux1fac*Aux1M[i][j-1]-aux1fac*Lax3M[i][j-1])*invspstep;
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Inf[i][j][2]=(-inf-aux1fac*Aux1M[i][j+1]-aux1fac*Lax3M[i][j+1])*invspstep;
	  }
	else if(Tc[i][j]==2)
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		Inf[i][j][0]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		Inf[i][j][1]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Inf[i][j][2]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Inf[i][j][3]=(inf+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
	  }
      }
  //Set Eff from Tc
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall, neighboring membrane
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		Eff[i][j][0]=(eff*invspstep+pPUMP*(Cpin1M[i+1][j])*invspstep);
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		Eff[i][j][1]=(eff*invspstep+pPUMP*(Cpin1M[i-1][j])*invspstep);
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Eff[i][j][2]=eff*invspstep+pPUMP*(Cpin1M[i][j+1])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Eff[i][j][3]=eff*invspstep+pPUMP*(Cpin1M[i][j-1])*invspstep;
	  }
	else if(Tc[i][j]==2)//membrane, neighboring wall
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		Eff[i][j][0]=(-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep);
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		Eff[i][j][1]=(-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep);
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Eff[i][j][2]=-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Eff[i][j][3]=-eff*invspstep-pPUMP*(Cpin1M[i][j])*invspstep;
	  }
      }
  //Set DiffAuxC from Tc
  double DauxCC=0.5*DauxC*tstep/(xstep*xstep);
  double DauxWW=0.5*DauxW*tstep/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall grid point: only diffuse towards other wall grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		DiffAuxC[i][j][0]=DauxWW;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		DiffAuxC[i][j][1]=DauxWW;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		DiffAuxC[i][j][2]=DauxWW;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		DiffAuxC[i][j][3]=DauxWW;
	  }
	else if(Tc[i][j]==1 || Tc[i][j]==2)//cell grid point: only diffuse towards other cell grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==1 || Tc[i+1][j]==2)
		DiffAuxC[i][j][0]=DauxCC;
	    if(i!=0)
	      if(Tc[i-1][j]==1 || Tc[i-1][j]==2)
		DiffAuxC[i][j][1]=DauxCC;
	    if(j!=m-1)
	      if(Tc[i][j+1]==1 || Tc[i][j+1]==2)
		DiffAuxC[i][j][2]=DauxCC;
	    if(j!=0)
	      if(Tc[i][j-1]==1 || Tc[i][j-1]==2)
		DiffAuxC[i][j][3]=DauxCC;
	  }
      }
//Now we need to adjust for getting a normal influx/efflux relation between root and shoot
#ifndef DEBUGMODE
#ifdef NORMALSHOOTROOTCONNECTION
  double auxinvirtlayer=1.5*0.8*0.5*0.8;//1;//0.5;//10;//0.5
#endif
#ifdef SHOOTROOTCUT
  double auxinvirtlayer=0.8*0.5*exp(-0.003*time);//0;
#endif
  int jbot;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	
	if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	  {
	    c=CellGrid[i][j-1];
	    if(Tct[i][j-1]==5 || Tct[i][j-1]==6 || Tct[i][j-1]==7)//efflux to shoot from epi/cort/endo
	      {		  
		jbot=c->CellMinJ-1;
		daux[i][j]=-0.1*Inf[i][jbot][2];
	      }
	    else if(Tct[i][j-1]==8 || Tct[i][j-1]==9 || Tct[i][j-1]==10 || Tct[i][j-1]==11)//influx from shoot to pericycle and vasculature
	      {
		jbot=c->CellMinJ-1;
		paux[i][j]=(Eff[i][jbot][2])*auxinvirtlayer;
	      }
	    break;
	  }
      }
#else
  for(i=0;i<n;i++)
    {
      for(j=m-1;j>=0;j--)
	{
	  if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	    {
	      paux[i][j]=4;//30;
	      break;
	    }
	}
    }
      
  //efflux from wall below bottommost cell
  j=0+bufsize;
  for(i=0;i<n;i++)
    {
      daux[i][j]=0.5;
    }

#endif
}


